# This file was auto-generated by Fern from our API Definition.

import os
import typing

import httpx
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.request_options import RequestOptions
from .environment import VellumEnvironment
from .raw_client import AsyncRawVellum, RawVellum
from .resources.ad_hoc.client import AdHocClient, AsyncAdHocClient
from .resources.container_images.client import AsyncContainerImagesClient, ContainerImagesClient
from .resources.deployments.client import AsyncDeploymentsClient, DeploymentsClient
from .resources.document_indexes.client import AsyncDocumentIndexesClient, DocumentIndexesClient
from .resources.documents.client import AsyncDocumentsClient, DocumentsClient
from .resources.environments.client import AsyncEnvironmentsClient, EnvironmentsClient
from .resources.events.client import AsyncEventsClient, EventsClient
from .resources.folder_entities.client import AsyncFolderEntitiesClient, FolderEntitiesClient
from .resources.integration_auth_configs.client import AsyncIntegrationAuthConfigsClient, IntegrationAuthConfigsClient
from .resources.integration_providers.client import AsyncIntegrationProvidersClient, IntegrationProvidersClient
from .resources.integrations.client import AsyncIntegrationsClient, IntegrationsClient
from .resources.metric_definitions.client import AsyncMetricDefinitionsClient, MetricDefinitionsClient
from .resources.ml_models.client import AsyncMlModelsClient, MlModelsClient
from .resources.organizations.client import AsyncOrganizationsClient, OrganizationsClient
from .resources.prompts.client import AsyncPromptsClient, PromptsClient
from .resources.sandboxes.client import AsyncSandboxesClient, SandboxesClient
from .resources.test_suite_runs.client import AsyncTestSuiteRunsClient, TestSuiteRunsClient
from .resources.test_suites.client import AsyncTestSuitesClient, TestSuitesClient
from .resources.workflow_deployments.client import AsyncWorkflowDeploymentsClient, WorkflowDeploymentsClient
from .resources.workflow_executions.client import AsyncWorkflowExecutionsClient, WorkflowExecutionsClient
from .resources.workflow_sandboxes.client import AsyncWorkflowSandboxesClient, WorkflowSandboxesClient
from .resources.workflows.client import AsyncWorkflowsClient, WorkflowsClient
from .resources.workspace_secrets.client import AsyncWorkspaceSecretsClient, WorkspaceSecretsClient
from .resources.workspaces.client import AsyncWorkspacesClient, WorkspacesClient
from .types.api_version_enum import ApiVersionEnum
from .types.code_execution_package import CodeExecutionPackage
from .types.code_execution_runtime import CodeExecutionRuntime
from .types.code_executor_input import CodeExecutorInput
from .types.code_executor_response import CodeExecutorResponse
from .types.execute_api_request_bearer_token import ExecuteApiRequestBearerToken
from .types.execute_api_request_body import ExecuteApiRequestBody
from .types.execute_api_request_headers_value import ExecuteApiRequestHeadersValue
from .types.execute_api_response import ExecuteApiResponse
from .types.execute_prompt_event import ExecutePromptEvent
from .types.execute_prompt_response import ExecutePromptResponse
from .types.execute_workflow_response import ExecuteWorkflowResponse
from .types.generate_options_request import GenerateOptionsRequest
from .types.generate_request import GenerateRequest
from .types.generate_response import GenerateResponse
from .types.generate_stream_response import GenerateStreamResponse
from .types.method_enum import MethodEnum
from .types.prompt_deployment_expand_meta_request import PromptDeploymentExpandMetaRequest
from .types.prompt_deployment_input_request import PromptDeploymentInputRequest
from .types.raw_prompt_execution_overrides_request import RawPromptExecutionOverridesRequest
from .types.search_request_options_request import SearchRequestOptionsRequest
from .types.search_response import SearchResponse
from .types.submit_completion_actual_request import SubmitCompletionActualRequest
from .types.submit_workflow_execution_actual_request import SubmitWorkflowExecutionActualRequest
from .types.vellum_variable_type import VellumVariableType
from .types.workflow_execution_event_type import WorkflowExecutionEventType
from .types.workflow_expand_meta_request import WorkflowExpandMetaRequest
from .types.workflow_request_input_request import WorkflowRequestInputRequest
from .types.workflow_stream_event import WorkflowStreamEvent

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class Vellum:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : VellumEnvironment
        The environment to use for requests from the client. from .environment import VellumEnvironment



        Defaults to VellumEnvironment.PRODUCTION



    api_version : typing.Optional[ApiVersionEnum]
    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default there is no timeout set, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from vellum import Vellum

    client = Vellum(
        api_version="YOUR_API_VERSION",
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: VellumEnvironment = VellumEnvironment.PRODUCTION,
        api_version: typing.Optional[ApiVersionEnum] = os.getenv("VELLUM_API_VERSION", "2025-07-30"),
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else None if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            environment=environment,
            api_version=api_version,
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._raw_client = RawVellum(client_wrapper=self._client_wrapper)
        self.integrations = IntegrationsClient(client_wrapper=self._client_wrapper)
        self.events = EventsClient(client_wrapper=self._client_wrapper)
        self.ad_hoc = AdHocClient(client_wrapper=self._client_wrapper)
        self.container_images = ContainerImagesClient(client_wrapper=self._client_wrapper)
        self.deployments = DeploymentsClient(client_wrapper=self._client_wrapper)
        self.document_indexes = DocumentIndexesClient(client_wrapper=self._client_wrapper)
        self.documents = DocumentsClient(client_wrapper=self._client_wrapper)
        self.environments = EnvironmentsClient(client_wrapper=self._client_wrapper)
        self.folder_entities = FolderEntitiesClient(client_wrapper=self._client_wrapper)
        self.integration_auth_configs = IntegrationAuthConfigsClient(client_wrapper=self._client_wrapper)
        self.integration_providers = IntegrationProvidersClient(client_wrapper=self._client_wrapper)
        self.metric_definitions = MetricDefinitionsClient(client_wrapper=self._client_wrapper)
        self.ml_models = MlModelsClient(client_wrapper=self._client_wrapper)
        self.organizations = OrganizationsClient(client_wrapper=self._client_wrapper)
        self.prompts = PromptsClient(client_wrapper=self._client_wrapper)
        self.sandboxes = SandboxesClient(client_wrapper=self._client_wrapper)
        self.test_suite_runs = TestSuiteRunsClient(client_wrapper=self._client_wrapper)
        self.test_suites = TestSuitesClient(client_wrapper=self._client_wrapper)
        self.workflow_deployments = WorkflowDeploymentsClient(client_wrapper=self._client_wrapper)
        self.workflow_executions = WorkflowExecutionsClient(client_wrapper=self._client_wrapper)
        self.workflow_sandboxes = WorkflowSandboxesClient(client_wrapper=self._client_wrapper)
        self.workflows = WorkflowsClient(client_wrapper=self._client_wrapper)
        self.workspace_secrets = WorkspaceSecretsClient(client_wrapper=self._client_wrapper)
        self.workspaces = WorkspacesClient(client_wrapper=self._client_wrapper)

    @property
    def with_raw_response(self) -> RawVellum:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVellum
        """
        return self._raw_client

    def execute_api(
        self,
        *,
        url: str,
        method: typing.Optional[MethodEnum] = OMIT,
        body: typing.Optional[ExecuteApiRequestBody] = OMIT,
        headers: typing.Optional[typing.Dict[str, ExecuteApiRequestHeadersValue]] = OMIT,
        bearer_token: typing.Optional[ExecuteApiRequestBearerToken] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecuteApiResponse:
        """
        Parameters
        ----------
        url : str

        method : typing.Optional[MethodEnum]

        body : typing.Optional[ExecuteApiRequestBody]

        headers : typing.Optional[typing.Dict[str, ExecuteApiRequestHeadersValue]]

        bearer_token : typing.Optional[ExecuteApiRequestBearerToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecuteApiResponse


        Examples
        --------
        from vellum import Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.execute_api(
            url="x",
        )
        """
        _response = self._raw_client.execute_api(
            url=url,
            method=method,
            body=body,
            headers=headers,
            bearer_token=bearer_token,
            request_options=request_options,
        )
        return _response.data

    def execute_code(
        self,
        *,
        code: str,
        runtime: CodeExecutionRuntime,
        input_values: typing.Sequence[CodeExecutorInput],
        packages: typing.Sequence[CodeExecutionPackage],
        output_type: VellumVariableType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CodeExecutorResponse:
        """
        Parameters
        ----------
        code : str

        runtime : CodeExecutionRuntime

        input_values : typing.Sequence[CodeExecutorInput]

        packages : typing.Sequence[CodeExecutionPackage]

        output_type : VellumVariableType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CodeExecutorResponse


        Examples
        --------
        from vellum import CodeExecutionPackage, StringInput, Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.execute_code(
            code="x",
            runtime="PYTHON_3_11_6",
            input_values=[
                StringInput(
                    name="x",
                    value="value",
                ),
                StringInput(
                    name="x",
                    value="value",
                ),
            ],
            packages=[
                CodeExecutionPackage(
                    version="version",
                    name="name",
                ),
                CodeExecutionPackage(
                    version="version",
                    name="name",
                ),
            ],
            output_type="STRING",
        )
        """
        _response = self._raw_client.execute_code(
            code=code,
            runtime=runtime,
            input_values=input_values,
            packages=packages,
            output_type=output_type,
            request_options=request_options,
        )
        return _response.data

    def execute_prompt(
        self,
        *,
        inputs: typing.Sequence[PromptDeploymentInputRequest],
        prompt_deployment_id: typing.Optional[str] = OMIT,
        prompt_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        expand_meta: typing.Optional[PromptDeploymentExpandMetaRequest] = OMIT,
        raw_overrides: typing.Optional[RawPromptExecutionOverridesRequest] = OMIT,
        expand_raw: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecutePromptResponse:
        """
        Executes a deployed Prompt and returns the result.

        Parameters
        ----------
        inputs : typing.Sequence[PromptDeploymentInputRequest]
            A list consisting of the Prompt Deployment's input variables and their values.

        prompt_deployment_id : typing.Optional[str]
            The ID of the Prompt Deployment. Must provide either this or prompt_deployment_name.

        prompt_deployment_name : typing.Optional[str]
            The unique name of the Prompt Deployment. Must provide either this or prompt_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Prompt Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        expand_meta : typing.Optional[PromptDeploymentExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this prompt execution in the API response. Corresponding values will be returned under the `meta` key of the API response.

        raw_overrides : typing.Optional[RawPromptExecutionOverridesRequest]
            Overrides for the raw API request sent to the model host. Combined with `expand_raw`, it can be used to access new features from models.

        expand_raw : typing.Optional[typing.Sequence[str]]
            A list of keys whose values you'd like to directly return from the JSON response of the model provider. Useful if you need lower-level info returned by model providers that Vellum would otherwise omit. Corresponding key/value pairs will be returned under the `raw` key of the API response.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecutePromptResponse


        Examples
        --------
        from vellum import StringInputRequest, Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.execute_prompt(
            inputs=[
                StringInputRequest(
                    name="x",
                    value="value",
                ),
                StringInputRequest(
                    name="x",
                    value="value",
                ),
            ],
        )
        """
        _response = self._raw_client.execute_prompt(
            inputs=inputs,
            prompt_deployment_id=prompt_deployment_id,
            prompt_deployment_name=prompt_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            expand_meta=expand_meta,
            raw_overrides=raw_overrides,
            expand_raw=expand_raw,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    def execute_prompt_stream(
        self,
        *,
        inputs: typing.Sequence[PromptDeploymentInputRequest],
        prompt_deployment_id: typing.Optional[str] = OMIT,
        prompt_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        expand_meta: typing.Optional[PromptDeploymentExpandMetaRequest] = OMIT,
        raw_overrides: typing.Optional[RawPromptExecutionOverridesRequest] = OMIT,
        expand_raw: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ExecutePromptEvent]:
        """
        Executes a deployed Prompt and streams back the results.

        Parameters
        ----------
        inputs : typing.Sequence[PromptDeploymentInputRequest]
            A list consisting of the Prompt Deployment's input variables and their values.

        prompt_deployment_id : typing.Optional[str]
            The ID of the Prompt Deployment. Must provide either this or prompt_deployment_name.

        prompt_deployment_name : typing.Optional[str]
            The unique name of the Prompt Deployment. Must provide either this or prompt_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Prompt Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        expand_meta : typing.Optional[PromptDeploymentExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this prompt execution in the API response. Corresponding values will be returned under the `meta` key of the API response.

        raw_overrides : typing.Optional[RawPromptExecutionOverridesRequest]
            Overrides for the raw API request sent to the model host. Combined with `expand_raw`, it can be used to access new features from models.

        expand_raw : typing.Optional[typing.Sequence[str]]
            A list of keys whose values you'd like to directly return from the JSON response of the model provider. Useful if you need lower-level info returned by model providers that Vellum would otherwise omit. Corresponding key/value pairs will be returned under the `raw` key of the API response.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[ExecutePromptEvent]


        Examples
        --------
        from vellum import StringInputRequest, Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        response = client.execute_prompt_stream(
            inputs=[
                StringInputRequest(
                    name="x",
                    value="value",
                ),
                StringInputRequest(
                    name="x",
                    value="value",
                ),
            ],
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.execute_prompt_stream(
            inputs=inputs,
            prompt_deployment_id=prompt_deployment_id,
            prompt_deployment_name=prompt_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            expand_meta=expand_meta,
            raw_overrides=raw_overrides,
            expand_raw=expand_raw,
            metadata=metadata,
            request_options=request_options,
        ) as r:
            yield from r.data

    def execute_workflow(
        self,
        *,
        inputs: typing.Sequence[WorkflowRequestInputRequest],
        expand_meta: typing.Optional[WorkflowExpandMetaRequest] = OMIT,
        workflow_deployment_id: typing.Optional[str] = OMIT,
        workflow_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        previous_execution_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecuteWorkflowResponse:
        """
        Executes a deployed Workflow and returns its outputs.

        Parameters
        ----------
        inputs : typing.Sequence[WorkflowRequestInputRequest]
            The list of inputs defined in the Workflow's Deployment with their corresponding values.

        expand_meta : typing.Optional[WorkflowExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this workflow execution in the API response. Corresponding values will be returned under the `execution_meta` key within NODE events in the response stream.

        workflow_deployment_id : typing.Optional[str]
            The ID of the Workflow Deployment. Must provide either this or workflow_deployment_name.

        workflow_deployment_name : typing.Optional[str]
            The name of the Workflow Deployment. Must provide either this or workflow_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Workflow Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        previous_execution_id : typing.Optional[str]
            The ID of a previous Workflow Execution to reference for initial State loading.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecuteWorkflowResponse


        Examples
        --------
        from vellum import Vellum, WorkflowRequestStringInputRequest

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.execute_workflow(
            inputs=[
                WorkflowRequestStringInputRequest(
                    name="x",
                    value="value",
                ),
                WorkflowRequestStringInputRequest(
                    name="x",
                    value="value",
                ),
            ],
        )
        """
        _response = self._raw_client.execute_workflow(
            inputs=inputs,
            expand_meta=expand_meta,
            workflow_deployment_id=workflow_deployment_id,
            workflow_deployment_name=workflow_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            metadata=metadata,
            previous_execution_id=previous_execution_id,
            request_options=request_options,
        )
        return _response.data

    def execute_workflow_stream(
        self,
        *,
        inputs: typing.Sequence[WorkflowRequestInputRequest],
        expand_meta: typing.Optional[WorkflowExpandMetaRequest] = OMIT,
        workflow_deployment_id: typing.Optional[str] = OMIT,
        workflow_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        event_types: typing.Optional[typing.Sequence[WorkflowExecutionEventType]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        previous_execution_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[WorkflowStreamEvent]:
        """
        Executes a deployed Workflow and streams back its results.

        Parameters
        ----------
        inputs : typing.Sequence[WorkflowRequestInputRequest]
            The list of inputs defined in the Workflow's Deployment with their corresponding values.

        expand_meta : typing.Optional[WorkflowExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this workflow execution in the API response. Corresponding values will be returned under the `execution_meta` key within NODE events in the response stream.

        workflow_deployment_id : typing.Optional[str]
            The ID of the Workflow Deployment. Must provide either this or workflow_deployment_name.

        workflow_deployment_name : typing.Optional[str]
            The name of the Workflow Deployment. Must provide either this or workflow_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Workflow Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        event_types : typing.Optional[typing.Sequence[WorkflowExecutionEventType]]
            Optionally specify which events you want to receive. Defaults to only WORKFLOW events. Note that the schema of non-WORKFLOW events is unstable and should be used with caution.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        previous_execution_id : typing.Optional[str]
            The ID of a previous Workflow Execution to reference for initial State loading.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[WorkflowStreamEvent]


        Examples
        --------
        from vellum import Vellum, WorkflowRequestStringInputRequest

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        response = client.execute_workflow_stream(
            inputs=[
                WorkflowRequestStringInputRequest(
                    name="x",
                    value="value",
                ),
                WorkflowRequestStringInputRequest(
                    name="x",
                    value="value",
                ),
            ],
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.execute_workflow_stream(
            inputs=inputs,
            expand_meta=expand_meta,
            workflow_deployment_id=workflow_deployment_id,
            workflow_deployment_name=workflow_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            event_types=event_types,
            metadata=metadata,
            previous_execution_id=previous_execution_id,
            request_options=request_options,
        ) as r:
            yield from r.data

    def generate(
        self,
        *,
        requests: typing.Sequence[GenerateRequest],
        deployment_id: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        options: typing.Optional[GenerateOptionsRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GenerateResponse:
        """
        Generate a completion using a previously defined deployment.

        Important: This endpoint is DEPRECATED and has been superseded by
        [execute-prompt](/api-reference/api-reference/execute-prompt).

        Parameters
        ----------
        requests : typing.Sequence[GenerateRequest]
            The generation request to make. Bulk requests are no longer supported, this field must be an array of length 1.

        deployment_id : typing.Optional[str]
            The ID of the deployment. Must provide either this or deployment_name.

        deployment_name : typing.Optional[str]
            The name of the deployment. Must provide either this or deployment_id.

        options : typing.Optional[GenerateOptionsRequest]
            Additional configuration that can be used to control what's included in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GenerateResponse


        Examples
        --------
        from vellum import GenerateRequest, Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.generate(
            requests=[
                GenerateRequest(
                    input_values={"input_values": {"key": "value"}},
                ),
                GenerateRequest(
                    input_values={"input_values": {"key": "value"}},
                ),
            ],
        )
        """
        _response = self._raw_client.generate(
            requests=requests,
            deployment_id=deployment_id,
            deployment_name=deployment_name,
            options=options,
            request_options=request_options,
        )
        return _response.data

    def generate_stream(
        self,
        *,
        requests: typing.Sequence[GenerateRequest],
        deployment_id: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        options: typing.Optional[GenerateOptionsRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[GenerateStreamResponse]:
        """
        Generate a stream of completions using a previously defined deployment.

        Important: This endpoint is DEPRECATED and has been superseded by
        [execute-prompt-stream](/api-reference/api-reference/execute-prompt-stream).

        Parameters
        ----------
        requests : typing.Sequence[GenerateRequest]
            The generation request to make. Bulk requests are no longer supported, this field must be an array of length 1.

        deployment_id : typing.Optional[str]
            The ID of the deployment. Must provide either this or deployment_name.

        deployment_name : typing.Optional[str]
            The name of the deployment. Must provide either this or deployment_id.

        options : typing.Optional[GenerateOptionsRequest]
            Additional configuration that can be used to control what's included in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[GenerateStreamResponse]


        Examples
        --------
        from vellum import GenerateRequest, Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        response = client.generate_stream(
            requests=[
                GenerateRequest(
                    input_values={"input_values": {"key": "value"}},
                ),
                GenerateRequest(
                    input_values={"input_values": {"key": "value"}},
                ),
            ],
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.generate_stream(
            requests=requests,
            deployment_id=deployment_id,
            deployment_name=deployment_name,
            options=options,
            request_options=request_options,
        ) as r:
            yield from r.data

    def search(
        self,
        *,
        query: str,
        index_id: typing.Optional[str] = OMIT,
        index_name: typing.Optional[str] = OMIT,
        options: typing.Optional[SearchRequestOptionsRequest] = OMIT,
        document_index: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchResponse:
        """
        Perform a search against a document index.

        Parameters
        ----------
        query : str
            The query to search for.

        index_id : typing.Optional[str]
            The ID of the index to search against. Must provide either this, index_name or document_index.

        index_name : typing.Optional[str]
            The name of the index to search against. Must provide either this, index_id or document_index.

        options : typing.Optional[SearchRequestOptionsRequest]
            Configuration options for the search.

        document_index : typing.Optional[str]
            Either the index name or index ID to search against. Must provide either this, index_id or index_name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchResponse


        Examples
        --------
        from vellum import Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.search(
            query="x",
        )
        """
        _response = self._raw_client.search(
            query=query,
            index_id=index_id,
            index_name=index_name,
            options=options,
            document_index=document_index,
            request_options=request_options,
        )
        return _response.data

    def submit_completion_actuals(
        self,
        *,
        actuals: typing.Sequence[SubmitCompletionActualRequest],
        deployment_id: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Used to submit feedback regarding the quality of previously generated completions.

        Parameters
        ----------
        actuals : typing.Sequence[SubmitCompletionActualRequest]
            Feedback regarding the quality of previously generated completions

        deployment_id : typing.Optional[str]
            The ID of the deployment. Must provide either this or deployment_name.

        deployment_name : typing.Optional[str]
            The name of the deployment. Must provide either this or deployment_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vellum import SubmitCompletionActualRequest, Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.submit_completion_actuals(
            actuals=[SubmitCompletionActualRequest(), SubmitCompletionActualRequest()],
        )
        """
        _response = self._raw_client.submit_completion_actuals(
            actuals=actuals,
            deployment_id=deployment_id,
            deployment_name=deployment_name,
            request_options=request_options,
        )
        return _response.data

    def submit_workflow_execution_actuals(
        self,
        *,
        actuals: typing.Sequence[SubmitWorkflowExecutionActualRequest],
        execution_id: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
            Used to submit feedback regarding the quality of previous workflow execution and its outputs.

            **Note:** Uses a base url of `https://predict.vellum.ai`.

        Parameters
        ----------
        actuals : typing.Sequence[SubmitWorkflowExecutionActualRequest]
            Feedback regarding the quality of an output on a previously executed workflow.

        execution_id : typing.Optional[str]
            The Vellum-generated ID of a previously executed workflow. Must provide either this or external_id.

        external_id : typing.Optional[str]
            The external ID that was originally provided by when executing the workflow, if applicable, that you'd now like to submit actuals for. Must provide either this or execution_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vellum import Vellum, WorkflowExecutionActualStringRequest

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.submit_workflow_execution_actuals(
            actuals=[
                WorkflowExecutionActualStringRequest(),
                WorkflowExecutionActualStringRequest(),
            ],
        )
        """
        _response = self._raw_client.submit_workflow_execution_actuals(
            actuals=actuals, execution_id=execution_id, external_id=external_id, request_options=request_options
        )
        return _response.data


class AsyncVellum:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : VellumEnvironment
        The environment to use for requests from the client. from .environment import VellumEnvironment



        Defaults to VellumEnvironment.PRODUCTION



    api_version : typing.Optional[ApiVersionEnum]
    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default there is no timeout set, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from vellum import AsyncVellum

    client = AsyncVellum(
        api_version="YOUR_API_VERSION",
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: VellumEnvironment = VellumEnvironment.PRODUCTION,
        api_version: typing.Optional[ApiVersionEnum] = os.getenv("VELLUM_API_VERSION", "2025-07-30"),
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else None if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            environment=environment,
            api_version=api_version,
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._raw_client = AsyncRawVellum(client_wrapper=self._client_wrapper)
        self.integrations = AsyncIntegrationsClient(client_wrapper=self._client_wrapper)
        self.events = AsyncEventsClient(client_wrapper=self._client_wrapper)
        self.ad_hoc = AsyncAdHocClient(client_wrapper=self._client_wrapper)
        self.container_images = AsyncContainerImagesClient(client_wrapper=self._client_wrapper)
        self.deployments = AsyncDeploymentsClient(client_wrapper=self._client_wrapper)
        self.document_indexes = AsyncDocumentIndexesClient(client_wrapper=self._client_wrapper)
        self.documents = AsyncDocumentsClient(client_wrapper=self._client_wrapper)
        self.environments = AsyncEnvironmentsClient(client_wrapper=self._client_wrapper)
        self.folder_entities = AsyncFolderEntitiesClient(client_wrapper=self._client_wrapper)
        self.integration_auth_configs = AsyncIntegrationAuthConfigsClient(client_wrapper=self._client_wrapper)
        self.integration_providers = AsyncIntegrationProvidersClient(client_wrapper=self._client_wrapper)
        self.metric_definitions = AsyncMetricDefinitionsClient(client_wrapper=self._client_wrapper)
        self.ml_models = AsyncMlModelsClient(client_wrapper=self._client_wrapper)
        self.organizations = AsyncOrganizationsClient(client_wrapper=self._client_wrapper)
        self.prompts = AsyncPromptsClient(client_wrapper=self._client_wrapper)
        self.sandboxes = AsyncSandboxesClient(client_wrapper=self._client_wrapper)
        self.test_suite_runs = AsyncTestSuiteRunsClient(client_wrapper=self._client_wrapper)
        self.test_suites = AsyncTestSuitesClient(client_wrapper=self._client_wrapper)
        self.workflow_deployments = AsyncWorkflowDeploymentsClient(client_wrapper=self._client_wrapper)
        self.workflow_executions = AsyncWorkflowExecutionsClient(client_wrapper=self._client_wrapper)
        self.workflow_sandboxes = AsyncWorkflowSandboxesClient(client_wrapper=self._client_wrapper)
        self.workflows = AsyncWorkflowsClient(client_wrapper=self._client_wrapper)
        self.workspace_secrets = AsyncWorkspaceSecretsClient(client_wrapper=self._client_wrapper)
        self.workspaces = AsyncWorkspacesClient(client_wrapper=self._client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVellum:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVellum
        """
        return self._raw_client

    async def execute_api(
        self,
        *,
        url: str,
        method: typing.Optional[MethodEnum] = OMIT,
        body: typing.Optional[ExecuteApiRequestBody] = OMIT,
        headers: typing.Optional[typing.Dict[str, ExecuteApiRequestHeadersValue]] = OMIT,
        bearer_token: typing.Optional[ExecuteApiRequestBearerToken] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecuteApiResponse:
        """
        Parameters
        ----------
        url : str

        method : typing.Optional[MethodEnum]

        body : typing.Optional[ExecuteApiRequestBody]

        headers : typing.Optional[typing.Dict[str, ExecuteApiRequestHeadersValue]]

        bearer_token : typing.Optional[ExecuteApiRequestBearerToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecuteApiResponse


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.execute_api(
                url="x",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_api(
            url=url,
            method=method,
            body=body,
            headers=headers,
            bearer_token=bearer_token,
            request_options=request_options,
        )
        return _response.data

    async def execute_code(
        self,
        *,
        code: str,
        runtime: CodeExecutionRuntime,
        input_values: typing.Sequence[CodeExecutorInput],
        packages: typing.Sequence[CodeExecutionPackage],
        output_type: VellumVariableType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CodeExecutorResponse:
        """
        Parameters
        ----------
        code : str

        runtime : CodeExecutionRuntime

        input_values : typing.Sequence[CodeExecutorInput]

        packages : typing.Sequence[CodeExecutionPackage]

        output_type : VellumVariableType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CodeExecutorResponse


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, CodeExecutionPackage, StringInput

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.execute_code(
                code="x",
                runtime="PYTHON_3_11_6",
                input_values=[
                    StringInput(
                        name="x",
                        value="value",
                    ),
                    StringInput(
                        name="x",
                        value="value",
                    ),
                ],
                packages=[
                    CodeExecutionPackage(
                        version="version",
                        name="name",
                    ),
                    CodeExecutionPackage(
                        version="version",
                        name="name",
                    ),
                ],
                output_type="STRING",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_code(
            code=code,
            runtime=runtime,
            input_values=input_values,
            packages=packages,
            output_type=output_type,
            request_options=request_options,
        )
        return _response.data

    async def execute_prompt(
        self,
        *,
        inputs: typing.Sequence[PromptDeploymentInputRequest],
        prompt_deployment_id: typing.Optional[str] = OMIT,
        prompt_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        expand_meta: typing.Optional[PromptDeploymentExpandMetaRequest] = OMIT,
        raw_overrides: typing.Optional[RawPromptExecutionOverridesRequest] = OMIT,
        expand_raw: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecutePromptResponse:
        """
        Executes a deployed Prompt and returns the result.

        Parameters
        ----------
        inputs : typing.Sequence[PromptDeploymentInputRequest]
            A list consisting of the Prompt Deployment's input variables and their values.

        prompt_deployment_id : typing.Optional[str]
            The ID of the Prompt Deployment. Must provide either this or prompt_deployment_name.

        prompt_deployment_name : typing.Optional[str]
            The unique name of the Prompt Deployment. Must provide either this or prompt_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Prompt Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        expand_meta : typing.Optional[PromptDeploymentExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this prompt execution in the API response. Corresponding values will be returned under the `meta` key of the API response.

        raw_overrides : typing.Optional[RawPromptExecutionOverridesRequest]
            Overrides for the raw API request sent to the model host. Combined with `expand_raw`, it can be used to access new features from models.

        expand_raw : typing.Optional[typing.Sequence[str]]
            A list of keys whose values you'd like to directly return from the JSON response of the model provider. Useful if you need lower-level info returned by model providers that Vellum would otherwise omit. Corresponding key/value pairs will be returned under the `raw` key of the API response.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecutePromptResponse


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, StringInputRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.execute_prompt(
                inputs=[
                    StringInputRequest(
                        name="x",
                        value="value",
                    ),
                    StringInputRequest(
                        name="x",
                        value="value",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_prompt(
            inputs=inputs,
            prompt_deployment_id=prompt_deployment_id,
            prompt_deployment_name=prompt_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            expand_meta=expand_meta,
            raw_overrides=raw_overrides,
            expand_raw=expand_raw,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    async def execute_prompt_stream(
        self,
        *,
        inputs: typing.Sequence[PromptDeploymentInputRequest],
        prompt_deployment_id: typing.Optional[str] = OMIT,
        prompt_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        expand_meta: typing.Optional[PromptDeploymentExpandMetaRequest] = OMIT,
        raw_overrides: typing.Optional[RawPromptExecutionOverridesRequest] = OMIT,
        expand_raw: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[ExecutePromptEvent]:
        """
        Executes a deployed Prompt and streams back the results.

        Parameters
        ----------
        inputs : typing.Sequence[PromptDeploymentInputRequest]
            A list consisting of the Prompt Deployment's input variables and their values.

        prompt_deployment_id : typing.Optional[str]
            The ID of the Prompt Deployment. Must provide either this or prompt_deployment_name.

        prompt_deployment_name : typing.Optional[str]
            The unique name of the Prompt Deployment. Must provide either this or prompt_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Prompt Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        expand_meta : typing.Optional[PromptDeploymentExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this prompt execution in the API response. Corresponding values will be returned under the `meta` key of the API response.

        raw_overrides : typing.Optional[RawPromptExecutionOverridesRequest]
            Overrides for the raw API request sent to the model host. Combined with `expand_raw`, it can be used to access new features from models.

        expand_raw : typing.Optional[typing.Sequence[str]]
            A list of keys whose values you'd like to directly return from the JSON response of the model provider. Useful if you need lower-level info returned by model providers that Vellum would otherwise omit. Corresponding key/value pairs will be returned under the `raw` key of the API response.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[ExecutePromptEvent]


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, StringInputRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.execute_prompt_stream(
                inputs=[
                    StringInputRequest(
                        name="x",
                        value="value",
                    ),
                    StringInputRequest(
                        name="x",
                        value="value",
                    ),
                ],
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.execute_prompt_stream(
            inputs=inputs,
            prompt_deployment_id=prompt_deployment_id,
            prompt_deployment_name=prompt_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            expand_meta=expand_meta,
            raw_overrides=raw_overrides,
            expand_raw=expand_raw,
            metadata=metadata,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def execute_workflow(
        self,
        *,
        inputs: typing.Sequence[WorkflowRequestInputRequest],
        expand_meta: typing.Optional[WorkflowExpandMetaRequest] = OMIT,
        workflow_deployment_id: typing.Optional[str] = OMIT,
        workflow_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        previous_execution_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecuteWorkflowResponse:
        """
        Executes a deployed Workflow and returns its outputs.

        Parameters
        ----------
        inputs : typing.Sequence[WorkflowRequestInputRequest]
            The list of inputs defined in the Workflow's Deployment with their corresponding values.

        expand_meta : typing.Optional[WorkflowExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this workflow execution in the API response. Corresponding values will be returned under the `execution_meta` key within NODE events in the response stream.

        workflow_deployment_id : typing.Optional[str]
            The ID of the Workflow Deployment. Must provide either this or workflow_deployment_name.

        workflow_deployment_name : typing.Optional[str]
            The name of the Workflow Deployment. Must provide either this or workflow_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Workflow Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        previous_execution_id : typing.Optional[str]
            The ID of a previous Workflow Execution to reference for initial State loading.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecuteWorkflowResponse


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, WorkflowRequestStringInputRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.execute_workflow(
                inputs=[
                    WorkflowRequestStringInputRequest(
                        name="x",
                        value="value",
                    ),
                    WorkflowRequestStringInputRequest(
                        name="x",
                        value="value",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_workflow(
            inputs=inputs,
            expand_meta=expand_meta,
            workflow_deployment_id=workflow_deployment_id,
            workflow_deployment_name=workflow_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            metadata=metadata,
            previous_execution_id=previous_execution_id,
            request_options=request_options,
        )
        return _response.data

    async def execute_workflow_stream(
        self,
        *,
        inputs: typing.Sequence[WorkflowRequestInputRequest],
        expand_meta: typing.Optional[WorkflowExpandMetaRequest] = OMIT,
        workflow_deployment_id: typing.Optional[str] = OMIT,
        workflow_deployment_name: typing.Optional[str] = OMIT,
        release_tag: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        event_types: typing.Optional[typing.Sequence[WorkflowExecutionEventType]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        previous_execution_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[WorkflowStreamEvent]:
        """
        Executes a deployed Workflow and streams back its results.

        Parameters
        ----------
        inputs : typing.Sequence[WorkflowRequestInputRequest]
            The list of inputs defined in the Workflow's Deployment with their corresponding values.

        expand_meta : typing.Optional[WorkflowExpandMetaRequest]
            An optionally specified configuration used to opt in to including additional metadata about this workflow execution in the API response. Corresponding values will be returned under the `execution_meta` key within NODE events in the response stream.

        workflow_deployment_id : typing.Optional[str]
            The ID of the Workflow Deployment. Must provide either this or workflow_deployment_name.

        workflow_deployment_name : typing.Optional[str]
            The name of the Workflow Deployment. Must provide either this or workflow_deployment_id.

        release_tag : typing.Optional[str]
            Optionally specify a release tag if you want to pin to a specific release of the Workflow Deployment

        external_id : typing.Optional[str]
            Optionally include a unique identifier for tracking purposes. Must be unique within a given Workspace.

        event_types : typing.Optional[typing.Sequence[WorkflowExecutionEventType]]
            Optionally specify which events you want to receive. Defaults to only WORKFLOW events. Note that the schema of non-WORKFLOW events is unstable and should be used with caution.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arbitrary JSON metadata associated with this request. Can be used to capture additional monitoring data such as user id, session id, etc. for future analysis.

        previous_execution_id : typing.Optional[str]
            The ID of a previous Workflow Execution to reference for initial State loading.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[WorkflowStreamEvent]


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, WorkflowRequestStringInputRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.execute_workflow_stream(
                inputs=[
                    WorkflowRequestStringInputRequest(
                        name="x",
                        value="value",
                    ),
                    WorkflowRequestStringInputRequest(
                        name="x",
                        value="value",
                    ),
                ],
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.execute_workflow_stream(
            inputs=inputs,
            expand_meta=expand_meta,
            workflow_deployment_id=workflow_deployment_id,
            workflow_deployment_name=workflow_deployment_name,
            release_tag=release_tag,
            external_id=external_id,
            event_types=event_types,
            metadata=metadata,
            previous_execution_id=previous_execution_id,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def generate(
        self,
        *,
        requests: typing.Sequence[GenerateRequest],
        deployment_id: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        options: typing.Optional[GenerateOptionsRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GenerateResponse:
        """
        Generate a completion using a previously defined deployment.

        Important: This endpoint is DEPRECATED and has been superseded by
        [execute-prompt](/api-reference/api-reference/execute-prompt).

        Parameters
        ----------
        requests : typing.Sequence[GenerateRequest]
            The generation request to make. Bulk requests are no longer supported, this field must be an array of length 1.

        deployment_id : typing.Optional[str]
            The ID of the deployment. Must provide either this or deployment_name.

        deployment_name : typing.Optional[str]
            The name of the deployment. Must provide either this or deployment_id.

        options : typing.Optional[GenerateOptionsRequest]
            Additional configuration that can be used to control what's included in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GenerateResponse


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, GenerateRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.generate(
                requests=[
                    GenerateRequest(
                        input_values={"input_values": {"key": "value"}},
                    ),
                    GenerateRequest(
                        input_values={"input_values": {"key": "value"}},
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.generate(
            requests=requests,
            deployment_id=deployment_id,
            deployment_name=deployment_name,
            options=options,
            request_options=request_options,
        )
        return _response.data

    async def generate_stream(
        self,
        *,
        requests: typing.Sequence[GenerateRequest],
        deployment_id: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        options: typing.Optional[GenerateOptionsRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[GenerateStreamResponse]:
        """
        Generate a stream of completions using a previously defined deployment.

        Important: This endpoint is DEPRECATED and has been superseded by
        [execute-prompt-stream](/api-reference/api-reference/execute-prompt-stream).

        Parameters
        ----------
        requests : typing.Sequence[GenerateRequest]
            The generation request to make. Bulk requests are no longer supported, this field must be an array of length 1.

        deployment_id : typing.Optional[str]
            The ID of the deployment. Must provide either this or deployment_name.

        deployment_name : typing.Optional[str]
            The name of the deployment. Must provide either this or deployment_id.

        options : typing.Optional[GenerateOptionsRequest]
            Additional configuration that can be used to control what's included in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[GenerateStreamResponse]


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, GenerateRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.generate_stream(
                requests=[
                    GenerateRequest(
                        input_values={"input_values": {"key": "value"}},
                    ),
                    GenerateRequest(
                        input_values={"input_values": {"key": "value"}},
                    ),
                ],
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.generate_stream(
            requests=requests,
            deployment_id=deployment_id,
            deployment_name=deployment_name,
            options=options,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def search(
        self,
        *,
        query: str,
        index_id: typing.Optional[str] = OMIT,
        index_name: typing.Optional[str] = OMIT,
        options: typing.Optional[SearchRequestOptionsRequest] = OMIT,
        document_index: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchResponse:
        """
        Perform a search against a document index.

        Parameters
        ----------
        query : str
            The query to search for.

        index_id : typing.Optional[str]
            The ID of the index to search against. Must provide either this, index_name or document_index.

        index_name : typing.Optional[str]
            The name of the index to search against. Must provide either this, index_id or document_index.

        options : typing.Optional[SearchRequestOptionsRequest]
            Configuration options for the search.

        document_index : typing.Optional[str]
            Either the index name or index ID to search against. Must provide either this, index_id or index_name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchResponse


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.search(
                query="x",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            query=query,
            index_id=index_id,
            index_name=index_name,
            options=options,
            document_index=document_index,
            request_options=request_options,
        )
        return _response.data

    async def submit_completion_actuals(
        self,
        *,
        actuals: typing.Sequence[SubmitCompletionActualRequest],
        deployment_id: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Used to submit feedback regarding the quality of previously generated completions.

        Parameters
        ----------
        actuals : typing.Sequence[SubmitCompletionActualRequest]
            Feedback regarding the quality of previously generated completions

        deployment_id : typing.Optional[str]
            The ID of the deployment. Must provide either this or deployment_name.

        deployment_name : typing.Optional[str]
            The name of the deployment. Must provide either this or deployment_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, SubmitCompletionActualRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.submit_completion_actuals(
                actuals=[
                    SubmitCompletionActualRequest(),
                    SubmitCompletionActualRequest(),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.submit_completion_actuals(
            actuals=actuals,
            deployment_id=deployment_id,
            deployment_name=deployment_name,
            request_options=request_options,
        )
        return _response.data

    async def submit_workflow_execution_actuals(
        self,
        *,
        actuals: typing.Sequence[SubmitWorkflowExecutionActualRequest],
        execution_id: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
            Used to submit feedback regarding the quality of previous workflow execution and its outputs.

            **Note:** Uses a base url of `https://predict.vellum.ai`.

        Parameters
        ----------
        actuals : typing.Sequence[SubmitWorkflowExecutionActualRequest]
            Feedback regarding the quality of an output on a previously executed workflow.

        execution_id : typing.Optional[str]
            The Vellum-generated ID of a previously executed workflow. Must provide either this or external_id.

        external_id : typing.Optional[str]
            The external ID that was originally provided by when executing the workflow, if applicable, that you'd now like to submit actuals for. Must provide either this or execution_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from vellum import AsyncVellum, WorkflowExecutionActualStringRequest

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.submit_workflow_execution_actuals(
                actuals=[
                    WorkflowExecutionActualStringRequest(),
                    WorkflowExecutionActualStringRequest(),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.submit_workflow_execution_actuals(
            actuals=actuals, execution_id=execution_id, external_id=external_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.
from .plugins.utils import load_runtime_plugins

load_runtime_plugins()

# isort: skip_file

from .client.types import (
    AdHocExecutePromptEvent,
    AdHocExpandMeta,
    AdHocFulfilledPromptExecutionMeta,
    AdHocInitiatedPromptExecutionMeta,
    AdHocRejectedPromptExecutionMeta,
    AdHocStreamingPromptExecutionMeta,
    AddOpenaiApiKeyEnum,
    ApiActorTypeEnum,
    ApiNodeResult,
    ApiNodeResultData,
    ApiRequestParentContext,
    ApiVersionEnum,
    ArrayChatMessageContent,
    ArrayChatMessageContentItem,
    ArrayChatMessageContentItemRequest,
    ArrayChatMessageContentRequest,
    ArrayInput,
    ArrayVellumValue,
    ArrayVellumValueRequest,
    AudioChatMessageContent,
    AudioChatMessageContentRequest,
    AudioInput,
    AudioInputRequest,
    AudioPromptBlock,
    AudioVellumValue,
    AudioVellumValueRequest,
    AuthTypeEnum,
    BaseOutput,
    BasicVectorizerIntfloatMultilingualE5Large,
    BasicVectorizerIntfloatMultilingualE5LargeRequest,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1Request,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1Request,
    BuildStatusEnum,
    ChatHistoryInput,
    ChatHistoryInputRequest,
    ChatHistoryVellumValue,
    ChatHistoryVellumValueRequest,
    ChatMessage,
    ChatMessageContent,
    ChatMessageContentRequest,
    ChatMessagePromptBlock,
    ChatMessageRequest,
    ChatMessageRole,
    CodeExecutionNodeArrayResult,
    CodeExecutionNodeChatHistoryResult,
    CodeExecutionNodeErrorResult,
    CodeExecutionNodeFunctionCallResult,
    CodeExecutionNodeJsonResult,
    CodeExecutionNodeNumberResult,
    CodeExecutionNodeResult,
    CodeExecutionNodeResultData,
    CodeExecutionNodeResultOutput,
    CodeExecutionNodeSearchResultsResult,
    CodeExecutionNodeStringResult,
    CodeExecutionPackage,
    CodeExecutionRuntime,
    CodeExecutorInput,
    CodeExecutorResponse,
    CodeExecutorSecretInput,
    CodeResourceDefinition,
    CompilePromptDeploymentExpandMetaRequest,
    CompilePromptMeta,
    ComponentsSchemasComposioExecuteToolRequest,
    ComponentsSchemasComposioExecuteToolResponse,
    ComponentsSchemasComposioIntegrationExecConfig,
    ComponentsSchemasComposioToolDefinition,
    ComponentsSchemasPdfSearchResultMetaSource,
    ComponentsSchemasPdfSearchResultMetaSourceRequest,
    ComponentsSchemasPromptVersionBuildConfigSandbox,
    ComponentsSchemasSlimComposioToolDefinition,
    ComposioExecuteToolRequest,
    ComposioExecuteToolResponse,
    ComposioIntegrationExecConfig,
    ComposioToolDefinition,
    ConditionCombinator,
    ConditionalNodeResult,
    ConditionalNodeResultData,
    ContainerImageBuildConfig,
    ContainerImageContainerImageTag,
    ContainerImageRead,
    CreateTestSuiteTestCaseRequest,
    CreateWorkflowEventRequest,
    DatasetRowPushRequest,
    DelimiterChunkerConfig,
    DelimiterChunkerConfigRequest,
    DelimiterChunking,
    DelimiterChunkingRequest,
    DeploymentHistoryItem,
    DeploymentProviderPayloadResponse,
    DeploymentProviderPayloadResponsePayload,
    DeploymentRead,
    DeploymentReleaseTagDeploymentHistoryItem,
    DeploymentReleaseTagRead,
    DeprecatedPromptRequestInput,
    DockerServiceToken,
    DocumentChatMessageContent,
    DocumentChatMessageContentRequest,
    DocumentDocumentToDocumentIndex,
    DocumentIndexChunking,
    DocumentIndexChunkingRequest,
    DocumentIndexIndexingConfig,
    DocumentIndexIndexingConfigRequest,
    DocumentIndexRead,
    DocumentInput,
    DocumentInputRequest,
    DocumentProcessingState,
    DocumentPromptBlock,
    DocumentRead,
    DocumentStatus,
    DocumentVellumValue,
    DocumentVellumValueRequest,
    EnrichedNormalizedCompletion,
    EntityStatus,
    EntityVisibility,
    EnvironmentDisplayConfig,
    EnvironmentEnum,
    EnvironmentRead,
    EphemeralPromptCacheConfig,
    EphemeralPromptCacheConfigTypeEnum,
    ErrorDetailResponse,
    ErrorInput,
    ErrorVellumValue,
    ErrorVellumValueRequest,
    EventCreateResponse,
    ExecuteApiRequestBearerToken,
    ExecuteApiRequestBody,
    ExecuteApiRequestHeadersValue,
    ExecuteApiResponse,
    ExecuteApiResponseJson,
    ExecutePromptEvent,
    ExecutePromptResponse,
    ExecuteWorkflowResponse,
    ExecuteWorkflowWorkflowResultEvent,
    ExecutionArrayVellumValue,
    ExecutionAudioVellumValue,
    ExecutionChatHistoryVellumValue,
    ExecutionDocumentVellumValue,
    ExecutionErrorVellumValue,
    ExecutionFunctionCallVellumValue,
    ExecutionImageVellumValue,
    ExecutionJsonVellumValue,
    ExecutionNumberVellumValue,
    ExecutionSearchResultsVellumValue,
    ExecutionStringVellumValue,
    ExecutionThinkingVellumValue,
    ExecutionVellumValue,
    ExecutionVideoVellumValue,
    ExternalInputDescriptor,
    ExternalParentContext,
    ExternalTestCaseExecution,
    ExternalTestCaseExecutionRequest,
    FastEmbedVectorizerBaaiBgeSmallEnV15,
    FastEmbedVectorizerBaaiBgeSmallEnV15Request,
    FinishReasonEnum,
    FolderEntity,
    FolderEntityDataset,
    FolderEntityDatasetData,
    FolderEntityDocumentIndex,
    FolderEntityDocumentIndexData,
    FolderEntityFolder,
    FolderEntityFolderData,
    FolderEntityPromptSandbox,
    FolderEntityPromptSandboxData,
    FolderEntityTestSuite,
    FolderEntityTestSuiteData,
    FolderEntityWorkflowSandbox,
    FolderEntityWorkflowSandboxData,
    FulfilledAdHocExecutePromptEvent,
    FulfilledExecutePromptEvent,
    FulfilledExecutePromptResponse,
    FulfilledExecuteWorkflowWorkflowResultEvent,
    FulfilledPromptExecutionMeta,
    FulfilledWorkflowNodeResultEvent,
    FunctionCall,
    FunctionCallChatMessageContent,
    FunctionCallChatMessageContentRequest,
    FunctionCallChatMessageContentValue,
    FunctionCallChatMessageContentValueRequest,
    FunctionCallInput,
    FunctionCallPromptBlock,
    FunctionCallRequest,
    FunctionCallVellumValue,
    FunctionCallVellumValueRequest,
    FunctionDefinition,
    GenerateOptionsRequest,
    GenerateRequest,
    GenerateResponse,
    GenerateResult,
    GenerateResultData,
    GenerateResultError,
    GenerateStreamResponse,
    GenerateStreamResult,
    GenerateStreamResultData,
    GoogleVertexAiVectorizerConfig,
    GoogleVertexAiVectorizerConfigRequest,
    GoogleVertexAiVectorizerGeminiEmbedding001,
    GoogleVertexAiVectorizerGeminiEmbedding001Request,
    GoogleVertexAiVectorizerTextEmbedding004,
    GoogleVertexAiVectorizerTextEmbedding004Request,
    GoogleVertexAiVectorizerTextMultilingualEmbedding002,
    GoogleVertexAiVectorizerTextMultilingualEmbedding002Request,
    HkunlpInstructorXlVectorizer,
    HkunlpInstructorXlVectorizerRequest,
    ImageChatMessageContent,
    ImageChatMessageContentRequest,
    ImageInput,
    ImageInputRequest,
    ImagePromptBlock,
    ImageVellumValue,
    ImageVellumValueRequest,
    IndexingConfigVectorizer,
    IndexingConfigVectorizerRequest,
    IndexingStateEnum,
    InitiatedAdHocExecutePromptEvent,
    InitiatedExecutePromptEvent,
    InitiatedPromptExecutionMeta,
    InitiatedWorkflowNodeResultEvent,
    InstructorVectorizerConfig,
    InstructorVectorizerConfigRequest,
    Integration,
    IntegrationAuthConfigIntegration,
    IntegrationAuthConfigIntegrationCredential,
    IntegrationCredentialAccessType,
    IntegrationName,
    IntegrationProvider,
    IntegrationRead,
    IntegrationTriggerContext,
    InvokedPort,
    IterationStateEnum,
    JinjaPromptBlock,
    JsonInput,
    JsonInputRequest,
    JsonVellumValue,
    JsonVellumValueRequest,
    LogicalOperator,
    LogprobsEnum,
    MapNodeResult,
    MapNodeResultData,
    MergeNodeResult,
    MergeNodeResultData,
    MetadataFilterConfigRequest,
    MetadataFilterRuleCombinator,
    MetadataFilterRuleRequest,
    MetadataFiltersRequest,
    MethodEnum,
    MetricDefinitionExecution,
    MetricDefinitionHistoryItem,
    MetricDefinitionInput,
    MetricNodeResult,
    MlModelRead,
    MlModelUsage,
    MlModelUsageWrapper,
    NamedScenarioInputAudioVariableValueRequest,
    NamedScenarioInputChatHistoryVariableValueRequest,
    NamedScenarioInputDocumentVariableValueRequest,
    NamedScenarioInputImageVariableValueRequest,
    NamedScenarioInputJsonVariableValueRequest,
    NamedScenarioInputRequest,
    NamedScenarioInputStringVariableValueRequest,
    NamedScenarioInputVideoVariableValueRequest,
    NamedTestCaseArrayVariableValue,
    NamedTestCaseArrayVariableValueRequest,
    NamedTestCaseAudioVariableValue,
    NamedTestCaseAudioVariableValueRequest,
    NamedTestCaseChatHistoryVariableValue,
    NamedTestCaseChatHistoryVariableValueRequest,
    NamedTestCaseDocumentVariableValue,
    NamedTestCaseDocumentVariableValueRequest,
    NamedTestCaseErrorVariableValue,
    NamedTestCaseErrorVariableValueRequest,
    NamedTestCaseFunctionCallVariableValue,
    NamedTestCaseFunctionCallVariableValueRequest,
    NamedTestCaseImageVariableValue,
    NamedTestCaseImageVariableValueRequest,
    NamedTestCaseJsonVariableValue,
    NamedTestCaseJsonVariableValueRequest,
    NamedTestCaseNumberVariableValue,
    NamedTestCaseNumberVariableValueRequest,
    NamedTestCaseSearchResultsVariableValue,
    NamedTestCaseSearchResultsVariableValueRequest,
    NamedTestCaseStringVariableValue,
    NamedTestCaseStringVariableValueRequest,
    NamedTestCaseVariableValue,
    NamedTestCaseVariableValueRequest,
    NamedTestCaseVideoVariableValue,
    NamedTestCaseVideoVariableValueRequest,
    NewMemberJoinBehaviorEnum,
    NodeExecutionFulfilledBody,
    NodeExecutionFulfilledEvent,
    NodeExecutionInitiatedBody,
    NodeExecutionInitiatedEvent,
    NodeExecutionPausedBody,
    NodeExecutionPausedEvent,
    NodeExecutionRejectedBody,
    NodeExecutionRejectedEvent,
    NodeExecutionResumedBody,
    NodeExecutionResumedEvent,
    NodeExecutionSpan,
    NodeExecutionSpanAttributes,
    NodeExecutionStreamingBody,
    NodeExecutionStreamingEvent,
    NodeInputCompiledArrayValue,
    NodeInputCompiledAudioValue,
    NodeInputCompiledChatHistoryValue,
    NodeInputCompiledDocumentValue,
    NodeInputCompiledErrorValue,
    NodeInputCompiledFunctionCallValue,
    NodeInputCompiledImageValue,
    NodeInputCompiledJsonValue,
    NodeInputCompiledNumberValue,
    NodeInputCompiledSearchResultsValue,
    NodeInputCompiledSecretValue,
    NodeInputCompiledStringValue,
    NodeInputCompiledVideoValue,
    NodeInputVariableCompiledValue,
    NodeOutputCompiledArrayValue,
    NodeOutputCompiledChatHistoryValue,
    NodeOutputCompiledErrorValue,
    NodeOutputCompiledFunctionCallValue,
    NodeOutputCompiledJsonValue,
    NodeOutputCompiledNumberValue,
    NodeOutputCompiledSearchResultsValue,
    NodeOutputCompiledStringValue,
    NodeOutputCompiledThinkingValue,
    NodeOutputCompiledValue,
    NodeParentContext,
    NormalizedLogProbs,
    NormalizedTokenLogProbs,
    NumberInput,
    NumberVellumValue,
    NumberVellumValueRequest,
    OpenAiVectorizerConfig,
    OpenAiVectorizerConfigRequest,
    OpenAiVectorizerTextEmbedding3Large,
    OpenAiVectorizerTextEmbedding3LargeRequest,
    OpenAiVectorizerTextEmbedding3Small,
    OpenAiVectorizerTextEmbedding3SmallRequest,
    OpenAiVectorizerTextEmbeddingAda002,
    OpenAiVectorizerTextEmbeddingAda002Request,
    OrganizationRead,
    PaginatedContainerImageReadList,
    PaginatedDeploymentReleaseTagReadList,
    PaginatedDocumentIndexReadList,
    PaginatedFolderEntityList,
    PaginatedSlimDeploymentReadList,
    PaginatedSlimDocumentList,
    PaginatedSlimIntegrationAuthConfigReadList,
    PaginatedSlimIntegrationReadList,
    PaginatedSlimToolDefinitionList,
    PaginatedSlimWorkflowDeploymentList,
    PaginatedTestSuiteRunExecutionList,
    PaginatedTestSuiteTestCaseList,
    PaginatedWorkflowDeploymentReleaseList,
    PaginatedWorkflowReleaseTagReadList,
    PaginatedWorkflowSandboxExampleList,
    ParentContext,
    PdfSearchResultMetaSource,
    PdfSearchResultMetaSourceRequest,
    PlainTextPromptBlock,
    Price,
    PrivateVectorizer,
    PrivateVectorizerRequest,
    ProcessingFailureReasonEnum,
    PromptBlock,
    PromptBlockState,
    PromptDeploymentExpandMetaRequest,
    PromptDeploymentInputRequest,
    PromptDeploymentParentContext,
    PromptDeploymentRelease,
    PromptDeploymentReleasePromptDeployment,
    PromptDeploymentReleasePromptVersion,
    PromptExecConfig,
    PromptExecutionMeta,
    PromptNodeExecutionMeta,
    PromptNodeResult,
    PromptNodeResultData,
    PromptOutput,
    PromptParameters,
    PromptPushResponse,
    PromptRequestAudioInput,
    PromptRequestChatHistoryInput,
    PromptRequestDocumentInput,
    PromptRequestImageInput,
    PromptRequestInput,
    PromptRequestJsonInput,
    PromptRequestStringInput,
    PromptRequestVideoInput,
    PromptSettings,
    PromptVersionBuildConfigSandbox,
    RawPromptExecutionOverridesRequest,
    ReductoChunkerConfig,
    ReductoChunkerConfigRequest,
    ReductoChunking,
    ReductoChunkingRequest,
    RejectedAdHocExecutePromptEvent,
    RejectedExecutePromptEvent,
    RejectedExecutePromptResponse,
    RejectedExecuteWorkflowWorkflowResultEvent,
    RejectedPromptExecutionMeta,
    RejectedWorkflowNodeResultEvent,
    ReleaseCreatedBy,
    ReleaseEnvironment,
    ReleaseReleaseTag,
    ReleaseReviewReviewer,
    ReleaseReviewState,
    ReleaseTagRelease,
    ReleaseTagSource,
    ReplaceTestSuiteTestCaseRequest,
    RichTextChildBlock,
    RichTextPromptBlock,
    SandboxScenario,
    ScenarioInput,
    ScenarioInputAudioVariableValue,
    ScenarioInputChatHistoryVariableValue,
    ScenarioInputDocumentVariableValue,
    ScenarioInputImageVariableValue,
    ScenarioInputJsonVariableValue,
    ScenarioInputStringVariableValue,
    ScenarioInputVideoVariableValue,
    ScheduledTriggerContext,
    SearchFiltersRequest,
    SearchNodeResult,
    SearchNodeResultData,
    SearchRequestOptionsRequest,
    SearchResponse,
    SearchResult,
    SearchResultDocument,
    SearchResultDocumentRequest,
    SearchResultMergingRequest,
    SearchResultMeta,
    SearchResultMetaRequest,
    SearchResultRequest,
    SearchResultsInput,
    SearchResultsVellumValue,
    SearchResultsVellumValueRequest,
    SearchWeightsRequest,
    SecretTypeEnum,
    SentenceChunkerConfig,
    SentenceChunkerConfigRequest,
    SentenceChunking,
    SentenceChunkingRequest,
    SlimComposioToolDefinition,
    SlimDeploymentRead,
    SlimDocument,
    SlimDocumentDocumentToDocumentIndex,
    SlimIntegrationAuthConfigRead,
    SlimIntegrationRead,
    SlimReleaseReview,
    SlimWorkflowDeployment,
    SlimWorkflowExecutionRead,
    SpanLink,
    SpanLinkTypeEnum,
    StreamingAdHocExecutePromptEvent,
    StreamingExecutePromptEvent,
    StreamingPromptExecutionMeta,
    StreamingWorkflowNodeResultEvent,
    StringChatMessageContent,
    StringChatMessageContentRequest,
    StringInput,
    StringInputRequest,
    StringVellumValue,
    StringVellumValueRequest,
    SubmitCompletionActualRequest,
    SubmitWorkflowExecutionActualRequest,
    SubworkflowNodeResult,
    SubworkflowNodeResultData,
    TemplatingNodeArrayResult,
    TemplatingNodeChatHistoryResult,
    TemplatingNodeErrorResult,
    TemplatingNodeFunctionCallResult,
    TemplatingNodeJsonResult,
    TemplatingNodeNumberResult,
    TemplatingNodeResult,
    TemplatingNodeResultData,
    TemplatingNodeResultOutput,
    TemplatingNodeSearchResultsResult,
    TemplatingNodeStringResult,
    TerminalNodeArrayResult,
    TerminalNodeChatHistoryResult,
    TerminalNodeErrorResult,
    TerminalNodeFunctionCallResult,
    TerminalNodeJsonResult,
    TerminalNodeNumberResult,
    TerminalNodeResult,
    TerminalNodeResultData,
    TerminalNodeResultOutput,
    TerminalNodeSearchResultsResult,
    TerminalNodeStringResult,
    TestCaseArrayVariableValue,
    TestCaseAudioVariableValue,
    TestCaseChatHistoryVariableValue,
    TestCaseDocumentVariableValue,
    TestCaseErrorVariableValue,
    TestCaseFunctionCallVariableValue,
    TestCaseImageVariableValue,
    TestCaseJsonVariableValue,
    TestCaseNumberVariableValue,
    TestCaseSearchResultsVariableValue,
    TestCaseStringVariableValue,
    TestCaseVariableValue,
    TestCaseVideoVariableValue,
    TestSuiteRunDeploymentReleaseTagExecConfig,
    TestSuiteRunDeploymentReleaseTagExecConfigData,
    TestSuiteRunDeploymentReleaseTagExecConfigDataRequest,
    TestSuiteRunDeploymentReleaseTagExecConfigRequest,
    TestSuiteRunExecConfig,
    TestSuiteRunExecConfigRequest,
    TestSuiteRunExecution,
    TestSuiteRunExecutionArrayOutput,
    TestSuiteRunExecutionChatHistoryOutput,
    TestSuiteRunExecutionErrorOutput,
    TestSuiteRunExecutionFunctionCallOutput,
    TestSuiteRunExecutionJsonOutput,
    TestSuiteRunExecutionMetricDefinition,
    TestSuiteRunExecutionMetricResult,
    TestSuiteRunExecutionNumberOutput,
    TestSuiteRunExecutionOutput,
    TestSuiteRunExecutionSearchResultsOutput,
    TestSuiteRunExecutionStringOutput,
    TestSuiteRunExternalExecConfig,
    TestSuiteRunExternalExecConfigData,
    TestSuiteRunExternalExecConfigDataRequest,
    TestSuiteRunExternalExecConfigRequest,
    TestSuiteRunMetricArrayOutput,
    TestSuiteRunMetricErrorOutput,
    TestSuiteRunMetricJsonOutput,
    TestSuiteRunMetricNumberOutput,
    TestSuiteRunMetricOutput,
    TestSuiteRunMetricStringOutput,
    TestSuiteRunProgress,
    TestSuiteRunPromptSandboxExecConfigDataRequest,
    TestSuiteRunPromptSandboxExecConfigRequest,
    TestSuiteRunPromptSandboxHistoryItemExecConfig,
    TestSuiteRunPromptSandboxHistoryItemExecConfigData,
    TestSuiteRunPromptSandboxHistoryItemExecConfigDataRequest,
    TestSuiteRunPromptSandboxHistoryItemExecConfigRequest,
    TestSuiteRunRead,
    TestSuiteRunState,
    TestSuiteRunTestSuite,
    TestSuiteRunWorkflowReleaseTagExecConfig,
    TestSuiteRunWorkflowReleaseTagExecConfigData,
    TestSuiteRunWorkflowReleaseTagExecConfigDataRequest,
    TestSuiteRunWorkflowReleaseTagExecConfigRequest,
    TestSuiteRunWorkflowSandboxExecConfigDataRequest,
    TestSuiteRunWorkflowSandboxExecConfigRequest,
    TestSuiteRunWorkflowSandboxHistoryItemExecConfig,
    TestSuiteRunWorkflowSandboxHistoryItemExecConfigData,
    TestSuiteRunWorkflowSandboxHistoryItemExecConfigDataRequest,
    TestSuiteRunWorkflowSandboxHistoryItemExecConfigRequest,
    TestSuiteTestCase,
    TestSuiteTestCaseBulkOperationRequest,
    TestSuiteTestCaseBulkResult,
    TestSuiteTestCaseCreateBulkOperationRequest,
    TestSuiteTestCaseCreatedBulkResult,
    TestSuiteTestCaseCreatedBulkResultData,
    TestSuiteTestCaseDeleteBulkOperationDataRequest,
    TestSuiteTestCaseDeleteBulkOperationRequest,
    TestSuiteTestCaseDeletedBulkResult,
    TestSuiteTestCaseDeletedBulkResultData,
    TestSuiteTestCaseRejectedBulkResult,
    TestSuiteTestCaseReplaceBulkOperationRequest,
    TestSuiteTestCaseReplacedBulkResult,
    TestSuiteTestCaseReplacedBulkResultData,
    TestSuiteTestCaseUpsertBulkOperationRequest,
    ThinkingVellumValue,
    ThinkingVellumValueRequest,
    TokenOverlappingWindowChunkerConfig,
    TokenOverlappingWindowChunkerConfigRequest,
    TokenOverlappingWindowChunking,
    TokenOverlappingWindowChunkingRequest,
    UnitEnum,
    UploadDocumentResponse,
    UpsertTestSuiteTestCaseRequest,
    VariablePromptBlock,
    VellumAudio,
    VellumAudioRequest,
    VellumCodeResourceDefinition,
    VellumDocument,
    VellumDocumentRequest,
    VellumError,
    VellumErrorCodeEnum,
    VellumErrorRequest,
    VellumImage,
    VellumImageRequest,
    VellumNodeExecutionEvent,
    VellumSdkError,
    VellumSdkErrorCodeEnum,
    VellumSecret,
    VellumSpan,
    VellumValue,
    VellumValueLogicalConditionGroupRequest,
    VellumValueLogicalConditionRequest,
    VellumValueLogicalExpressionRequest,
    VellumValueRequest,
    VellumVariable,
    VellumVariableExtensions,
    VellumVariableType,
    VellumVideo,
    VellumVideoRequest,
    VellumWorkflowExecutionEvent,
    VideoChatMessageContent,
    VideoChatMessageContentRequest,
    VideoInput,
    VideoInputRequest,
    VideoPromptBlock,
    VideoVellumValue,
    VideoVellumValueRequest,
    WorkflowDeploymentDisplayData,
    WorkflowDeploymentEventExecutionsResponse,
    WorkflowDeploymentHistoryItem,
    WorkflowDeploymentParentContext,
    WorkflowDeploymentRead,
    WorkflowDeploymentRelease,
    WorkflowDeploymentReleaseWorkflowDeployment,
    WorkflowDeploymentReleaseWorkflowVersion,
    WorkflowDisplayIcon,
    WorkflowError,
    WorkflowEvent,
    WorkflowEventError,
    WorkflowEventExecutionRead,
    WorkflowExecutionActual,
    WorkflowExecutionActualChatHistoryRequest,
    WorkflowExecutionActualJsonRequest,
    WorkflowExecutionActualStringRequest,
    WorkflowExecutionDetail,
    WorkflowExecutionEventErrorCode,
    WorkflowExecutionEventType,
    WorkflowExecutionFulfilledBody,
    WorkflowExecutionFulfilledEvent,
    WorkflowExecutionInitiatedBody,
    WorkflowExecutionInitiatedEvent,
    WorkflowExecutionNodeResultEvent,
    WorkflowExecutionPausedBody,
    WorkflowExecutionPausedEvent,
    WorkflowExecutionRejectedBody,
    WorkflowExecutionRejectedEvent,
    WorkflowExecutionResumedBody,
    WorkflowExecutionResumedEvent,
    WorkflowExecutionSnapshottedBody,
    WorkflowExecutionSnapshottedEvent,
    WorkflowExecutionSpan,
    WorkflowExecutionSpanAttributes,
    WorkflowExecutionStreamingBody,
    WorkflowExecutionStreamingEvent,
    WorkflowExecutionUsageCalculationError,
    WorkflowExecutionUsageCalculationErrorCodeEnum,
    WorkflowExecutionUsageCalculationFulfilledBody,
    WorkflowExecutionUsageResult,
    WorkflowExecutionViewOnlineEvalMetricResult,
    WorkflowExecutionWorkflowResultEvent,
    WorkflowExpandMetaRequest,
    WorkflowInitializationError,
    WorkflowInput,
    WorkflowNodeResultData,
    WorkflowNodeResultEvent,
    WorkflowNodeResultEventState,
    WorkflowOutput,
    WorkflowOutputArray,
    WorkflowOutputChatHistory,
    WorkflowOutputError,
    WorkflowOutputFunctionCall,
    WorkflowOutputImage,
    WorkflowOutputJson,
    WorkflowOutputNumber,
    WorkflowOutputSearchResults,
    WorkflowOutputString,
    WorkflowParentContext,
    WorkflowPushDeploymentConfigRequest,
    WorkflowPushExecConfig,
    WorkflowPushResponse,
    WorkflowReleaseTagRead,
    WorkflowReleaseTagWorkflowDeploymentHistoryItem,
    WorkflowRequestAudioInputRequest,
    WorkflowRequestChatHistoryInputRequest,
    WorkflowRequestDocumentInputRequest,
    WorkflowRequestImageInputRequest,
    WorkflowRequestInputRequest,
    WorkflowRequestJsonInputRequest,
    WorkflowRequestNumberInputRequest,
    WorkflowRequestStringInputRequest,
    WorkflowRequestVideoInputRequest,
    WorkflowResolvedState,
    WorkflowResultEvent,
    WorkflowResultEventOutputData,
    WorkflowResultEventOutputDataArray,
    WorkflowResultEventOutputDataChatHistory,
    WorkflowResultEventOutputDataError,
    WorkflowResultEventOutputDataFunctionCall,
    WorkflowResultEventOutputDataJson,
    WorkflowResultEventOutputDataNumber,
    WorkflowResultEventOutputDataSearchResults,
    WorkflowResultEventOutputDataString,
    WorkflowSandboxDisplayData,
    WorkflowSandboxExample,
    WorkflowSandboxParentContext,
    WorkflowStreamEvent,
    WorkspaceDisplayConfig,
    WorkspaceRead,
    WorkspaceSecretRead,
)
from .errors import (
    BadRequestError,
    ForbiddenError,
    InternalServerError,
    NotFoundError,
    TooManyRequestsError,
    UnauthorizedError,
)
from .resources import (
    DeploymentsListRequestStatus,
    DocumentIndexesListRequestStatus,
    FolderEntitiesListRequestEntityStatus,
    ListDeploymentReleaseTagsRequestSource,
    ListWorkflowReleaseTagsRequestSource,
    ListWorkflowSandboxExamplesRequestTag,
    WorkflowDeploymentsListRequestStatus,
    ad_hoc,
    container_images,
    deployments,
    document_indexes,
    documents,
    environments,
    events,
    folder_entities,
    integration_auth_configs,
    integration_providers,
    integrations,
    metric_definitions,
    ml_models,
    organizations,
    prompts,
    sandboxes,
    test_suite_runs,
    test_suites,
    workflow_deployments,
    workflow_executions,
    workflow_sandboxes,
    workflows,
    workspace_secrets,
    workspaces,
)
from .client import AsyncVellum, Vellum
from .environment import VellumEnvironment
from .version import __version__

__all__ = [
    "AdHocExecutePromptEvent",
    "AdHocExpandMeta",
    "AdHocFulfilledPromptExecutionMeta",
    "AdHocInitiatedPromptExecutionMeta",
    "AdHocRejectedPromptExecutionMeta",
    "AdHocStreamingPromptExecutionMeta",
    "AddOpenaiApiKeyEnum",
    "ApiActorTypeEnum",
    "ApiNodeResult",
    "ApiNodeResultData",
    "ApiRequestParentContext",
    "ApiVersionEnum",
    "ArrayChatMessageContent",
    "ArrayChatMessageContentItem",
    "ArrayChatMessageContentItemRequest",
    "ArrayChatMessageContentRequest",
    "ArrayInput",
    "ArrayVellumValue",
    "ArrayVellumValueRequest",
    "AsyncVellum",
    "AudioChatMessageContent",
    "AudioChatMessageContentRequest",
    "AudioInput",
    "AudioInputRequest",
    "AudioPromptBlock",
    "AudioVellumValue",
    "AudioVellumValueRequest",
    "AuthTypeEnum",
    "BadRequestError",
    "BaseOutput",
    "BasicVectorizerIntfloatMultilingualE5Large",
    "BasicVectorizerIntfloatMultilingualE5LargeRequest",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1Request",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1Request",
    "BuildStatusEnum",
    "ChatHistoryInput",
    "ChatHistoryInputRequest",
    "ChatHistoryVellumValue",
    "ChatHistoryVellumValueRequest",
    "ChatMessage",
    "ChatMessageContent",
    "ChatMessageContentRequest",
    "ChatMessagePromptBlock",
    "ChatMessageRequest",
    "ChatMessageRole",
    "CodeExecutionNodeArrayResult",
    "CodeExecutionNodeChatHistoryResult",
    "CodeExecutionNodeErrorResult",
    "CodeExecutionNodeFunctionCallResult",
    "CodeExecutionNodeJsonResult",
    "CodeExecutionNodeNumberResult",
    "CodeExecutionNodeResult",
    "CodeExecutionNodeResultData",
    "CodeExecutionNodeResultOutput",
    "CodeExecutionNodeSearchResultsResult",
    "CodeExecutionNodeStringResult",
    "CodeExecutionPackage",
    "CodeExecutionRuntime",
    "CodeExecutorInput",
    "CodeExecutorResponse",
    "CodeExecutorSecretInput",
    "CodeResourceDefinition",
    "CompilePromptDeploymentExpandMetaRequest",
    "CompilePromptMeta",
    "ComponentsSchemasComposioExecuteToolRequest",
    "ComponentsSchemasComposioExecuteToolResponse",
    "ComponentsSchemasComposioIntegrationExecConfig",
    "ComponentsSchemasComposioToolDefinition",
    "ComponentsSchemasPdfSearchResultMetaSource",
    "ComponentsSchemasPdfSearchResultMetaSourceRequest",
    "ComponentsSchemasPromptVersionBuildConfigSandbox",
    "ComponentsSchemasSlimComposioToolDefinition",
    "ComposioExecuteToolRequest",
    "ComposioExecuteToolResponse",
    "ComposioIntegrationExecConfig",
    "ComposioToolDefinition",
    "ConditionCombinator",
    "ConditionalNodeResult",
    "ConditionalNodeResultData",
    "ContainerImageBuildConfig",
    "ContainerImageContainerImageTag",
    "ContainerImageRead",
    "CreateTestSuiteTestCaseRequest",
    "CreateWorkflowEventRequest",
    "DatasetRowPushRequest",
    "DelimiterChunkerConfig",
    "DelimiterChunkerConfigRequest",
    "DelimiterChunking",
    "DelimiterChunkingRequest",
    "DeploymentHistoryItem",
    "DeploymentProviderPayloadResponse",
    "DeploymentProviderPayloadResponsePayload",
    "DeploymentRead",
    "DeploymentReleaseTagDeploymentHistoryItem",
    "DeploymentReleaseTagRead",
    "DeploymentsListRequestStatus",
    "DeprecatedPromptRequestInput",
    "DockerServiceToken",
    "DocumentChatMessageContent",
    "DocumentChatMessageContentRequest",
    "DocumentDocumentToDocumentIndex",
    "DocumentIndexChunking",
    "DocumentIndexChunkingRequest",
    "DocumentIndexIndexingConfig",
    "DocumentIndexIndexingConfigRequest",
    "DocumentIndexRead",
    "DocumentIndexesListRequestStatus",
    "DocumentInput",
    "DocumentInputRequest",
    "DocumentProcessingState",
    "DocumentPromptBlock",
    "DocumentRead",
    "DocumentStatus",
    "DocumentVellumValue",
    "DocumentVellumValueRequest",
    "EnrichedNormalizedCompletion",
    "EntityStatus",
    "EntityVisibility",
    "EnvironmentDisplayConfig",
    "EnvironmentEnum",
    "EnvironmentRead",
    "EphemeralPromptCacheConfig",
    "EphemeralPromptCacheConfigTypeEnum",
    "ErrorDetailResponse",
    "ErrorInput",
    "ErrorVellumValue",
    "ErrorVellumValueRequest",
    "EventCreateResponse",
    "ExecuteApiRequestBearerToken",
    "ExecuteApiRequestBody",
    "ExecuteApiRequestHeadersValue",
    "ExecuteApiResponse",
    "ExecuteApiResponseJson",
    "ExecutePromptEvent",
    "ExecutePromptResponse",
    "ExecuteWorkflowResponse",
    "ExecuteWorkflowWorkflowResultEvent",
    "ExecutionArrayVellumValue",
    "ExecutionAudioVellumValue",
    "ExecutionChatHistoryVellumValue",
    "ExecutionDocumentVellumValue",
    "ExecutionErrorVellumValue",
    "ExecutionFunctionCallVellumValue",
    "ExecutionImageVellumValue",
    "ExecutionJsonVellumValue",
    "ExecutionNumberVellumValue",
    "ExecutionSearchResultsVellumValue",
    "ExecutionStringVellumValue",
    "ExecutionThinkingVellumValue",
    "ExecutionVellumValue",
    "ExecutionVideoVellumValue",
    "ExternalInputDescriptor",
    "ExternalParentContext",
    "ExternalTestCaseExecution",
    "ExternalTestCaseExecutionRequest",
    "FastEmbedVectorizerBaaiBgeSmallEnV15",
    "FastEmbedVectorizerBaaiBgeSmallEnV15Request",
    "FinishReasonEnum",
    "FolderEntitiesListRequestEntityStatus",
    "FolderEntity",
    "FolderEntityDataset",
    "FolderEntityDatasetData",
    "FolderEntityDocumentIndex",
    "FolderEntityDocumentIndexData",
    "FolderEntityFolder",
    "FolderEntityFolderData",
    "FolderEntityPromptSandbox",
    "FolderEntityPromptSandboxData",
    "FolderEntityTestSuite",
    "FolderEntityTestSuiteData",
    "FolderEntityWorkflowSandbox",
    "FolderEntityWorkflowSandboxData",
    "ForbiddenError",
    "FulfilledAdHocExecutePromptEvent",
    "FulfilledExecutePromptEvent",
    "FulfilledExecutePromptResponse",
    "FulfilledExecuteWorkflowWorkflowResultEvent",
    "FulfilledPromptExecutionMeta",
    "FulfilledWorkflowNodeResultEvent",
    "FunctionCall",
    "FunctionCallChatMessageContent",
    "FunctionCallChatMessageContentRequest",
    "FunctionCallChatMessageContentValue",
    "FunctionCallChatMessageContentValueRequest",
    "FunctionCallInput",
    "FunctionCallPromptBlock",
    "FunctionCallRequest",
    "FunctionCallVellumValue",
    "FunctionCallVellumValueRequest",
    "FunctionDefinition",
    "GenerateOptionsRequest",
    "GenerateRequest",
    "GenerateResponse",
    "GenerateResult",
    "GenerateResultData",
    "GenerateResultError",
    "GenerateStreamResponse",
    "GenerateStreamResult",
    "GenerateStreamResultData",
    "GoogleVertexAiVectorizerConfig",
    "GoogleVertexAiVectorizerConfigRequest",
    "GoogleVertexAiVectorizerGeminiEmbedding001",
    "GoogleVertexAiVectorizerGeminiEmbedding001Request",
    "GoogleVertexAiVectorizerTextEmbedding004",
    "GoogleVertexAiVectorizerTextEmbedding004Request",
    "GoogleVertexAiVectorizerTextMultilingualEmbedding002",
    "GoogleVertexAiVectorizerTextMultilingualEmbedding002Request",
    "HkunlpInstructorXlVectorizer",
    "HkunlpInstructorXlVectorizerRequest",
    "ImageChatMessageContent",
    "ImageChatMessageContentRequest",
    "ImageInput",
    "ImageInputRequest",
    "ImagePromptBlock",
    "ImageVellumValue",
    "ImageVellumValueRequest",
    "IndexingConfigVectorizer",
    "IndexingConfigVectorizerRequest",
    "IndexingStateEnum",
    "InitiatedAdHocExecutePromptEvent",
    "InitiatedExecutePromptEvent",
    "InitiatedPromptExecutionMeta",
    "InitiatedWorkflowNodeResultEvent",
    "InstructorVectorizerConfig",
    "InstructorVectorizerConfigRequest",
    "Integration",
    "IntegrationAuthConfigIntegration",
    "IntegrationAuthConfigIntegrationCredential",
    "IntegrationCredentialAccessType",
    "IntegrationName",
    "IntegrationProvider",
    "IntegrationRead",
    "IntegrationTriggerContext",
    "InternalServerError",
    "InvokedPort",
    "IterationStateEnum",
    "JinjaPromptBlock",
    "JsonInput",
    "JsonInputRequest",
    "JsonVellumValue",
    "JsonVellumValueRequest",
    "ListDeploymentReleaseTagsRequestSource",
    "ListWorkflowReleaseTagsRequestSource",
    "ListWorkflowSandboxExamplesRequestTag",
    "LogicalOperator",
    "LogprobsEnum",
    "MapNodeResult",
    "MapNodeResultData",
    "MergeNodeResult",
    "MergeNodeResultData",
    "MetadataFilterConfigRequest",
    "MetadataFilterRuleCombinator",
    "MetadataFilterRuleRequest",
    "MetadataFiltersRequest",
    "MethodEnum",
    "MetricDefinitionExecution",
    "MetricDefinitionHistoryItem",
    "MetricDefinitionInput",
    "MetricNodeResult",
    "MlModelRead",
    "MlModelUsage",
    "MlModelUsageWrapper",
    "NamedScenarioInputAudioVariableValueRequest",
    "NamedScenarioInputChatHistoryVariableValueRequest",
    "NamedScenarioInputDocumentVariableValueRequest",
    "NamedScenarioInputImageVariableValueRequest",
    "NamedScenarioInputJsonVariableValueRequest",
    "NamedScenarioInputRequest",
    "NamedScenarioInputStringVariableValueRequest",
    "NamedScenarioInputVideoVariableValueRequest",
    "NamedTestCaseArrayVariableValue",
    "NamedTestCaseArrayVariableValueRequest",
    "NamedTestCaseAudioVariableValue",
    "NamedTestCaseAudioVariableValueRequest",
    "NamedTestCaseChatHistoryVariableValue",
    "NamedTestCaseChatHistoryVariableValueRequest",
    "NamedTestCaseDocumentVariableValue",
    "NamedTestCaseDocumentVariableValueRequest",
    "NamedTestCaseErrorVariableValue",
    "NamedTestCaseErrorVariableValueRequest",
    "NamedTestCaseFunctionCallVariableValue",
    "NamedTestCaseFunctionCallVariableValueRequest",
    "NamedTestCaseImageVariableValue",
    "NamedTestCaseImageVariableValueRequest",
    "NamedTestCaseJsonVariableValue",
    "NamedTestCaseJsonVariableValueRequest",
    "NamedTestCaseNumberVariableValue",
    "NamedTestCaseNumberVariableValueRequest",
    "NamedTestCaseSearchResultsVariableValue",
    "NamedTestCaseSearchResultsVariableValueRequest",
    "NamedTestCaseStringVariableValue",
    "NamedTestCaseStringVariableValueRequest",
    "NamedTestCaseVariableValue",
    "NamedTestCaseVariableValueRequest",
    "NamedTestCaseVideoVariableValue",
    "NamedTestCaseVideoVariableValueRequest",
    "NewMemberJoinBehaviorEnum",
    "NodeExecutionFulfilledBody",
    "NodeExecutionFulfilledEvent",
    "NodeExecutionInitiatedBody",
    "NodeExecutionInitiatedEvent",
    "NodeExecutionPausedBody",
    "NodeExecutionPausedEvent",
    "NodeExecutionRejectedBody",
    "NodeExecutionRejectedEvent",
    "NodeExecutionResumedBody",
    "NodeExecutionResumedEvent",
    "NodeExecutionSpan",
    "NodeExecutionSpanAttributes",
    "NodeExecutionStreamingBody",
    "NodeExecutionStreamingEvent",
    "NodeInputCompiledArrayValue",
    "NodeInputCompiledAudioValue",
    "NodeInputCompiledChatHistoryValue",
    "NodeInputCompiledDocumentValue",
    "NodeInputCompiledErrorValue",
    "NodeInputCompiledFunctionCallValue",
    "NodeInputCompiledImageValue",
    "NodeInputCompiledJsonValue",
    "NodeInputCompiledNumberValue",
    "NodeInputCompiledSearchResultsValue",
    "NodeInputCompiledSecretValue",
    "NodeInputCompiledStringValue",
    "NodeInputCompiledVideoValue",
    "NodeInputVariableCompiledValue",
    "NodeOutputCompiledArrayValue",
    "NodeOutputCompiledChatHistoryValue",
    "NodeOutputCompiledErrorValue",
    "NodeOutputCompiledFunctionCallValue",
    "NodeOutputCompiledJsonValue",
    "NodeOutputCompiledNumberValue",
    "NodeOutputCompiledSearchResultsValue",
    "NodeOutputCompiledStringValue",
    "NodeOutputCompiledThinkingValue",
    "NodeOutputCompiledValue",
    "NodeParentContext",
    "NormalizedLogProbs",
    "NormalizedTokenLogProbs",
    "NotFoundError",
    "NumberInput",
    "NumberVellumValue",
    "NumberVellumValueRequest",
    "OpenAiVectorizerConfig",
    "OpenAiVectorizerConfigRequest",
    "OpenAiVectorizerTextEmbedding3Large",
    "OpenAiVectorizerTextEmbedding3LargeRequest",
    "OpenAiVectorizerTextEmbedding3Small",
    "OpenAiVectorizerTextEmbedding3SmallRequest",
    "OpenAiVectorizerTextEmbeddingAda002",
    "OpenAiVectorizerTextEmbeddingAda002Request",
    "OrganizationRead",
    "PaginatedContainerImageReadList",
    "PaginatedDeploymentReleaseTagReadList",
    "PaginatedDocumentIndexReadList",
    "PaginatedFolderEntityList",
    "PaginatedSlimDeploymentReadList",
    "PaginatedSlimDocumentList",
    "PaginatedSlimIntegrationAuthConfigReadList",
    "PaginatedSlimIntegrationReadList",
    "PaginatedSlimToolDefinitionList",
    "PaginatedSlimWorkflowDeploymentList",
    "PaginatedTestSuiteRunExecutionList",
    "PaginatedTestSuiteTestCaseList",
    "PaginatedWorkflowDeploymentReleaseList",
    "PaginatedWorkflowReleaseTagReadList",
    "PaginatedWorkflowSandboxExampleList",
    "ParentContext",
    "PdfSearchResultMetaSource",
    "PdfSearchResultMetaSourceRequest",
    "PlainTextPromptBlock",
    "Price",
    "PrivateVectorizer",
    "PrivateVectorizerRequest",
    "ProcessingFailureReasonEnum",
    "PromptBlock",
    "PromptBlockState",
    "PromptDeploymentExpandMetaRequest",
    "PromptDeploymentInputRequest",
    "PromptDeploymentParentContext",
    "PromptDeploymentRelease",
    "PromptDeploymentReleasePromptDeployment",
    "PromptDeploymentReleasePromptVersion",
    "PromptExecConfig",
    "PromptExecutionMeta",
    "PromptNodeExecutionMeta",
    "PromptNodeResult",
    "PromptNodeResultData",
    "PromptOutput",
    "PromptParameters",
    "PromptPushResponse",
    "PromptRequestAudioInput",
    "PromptRequestChatHistoryInput",
    "PromptRequestDocumentInput",
    "PromptRequestImageInput",
    "PromptRequestInput",
    "PromptRequestJsonInput",
    "PromptRequestStringInput",
    "PromptRequestVideoInput",
    "PromptSettings",
    "PromptVersionBuildConfigSandbox",
    "RawPromptExecutionOverridesRequest",
    "ReductoChunkerConfig",
    "ReductoChunkerConfigRequest",
    "ReductoChunking",
    "ReductoChunkingRequest",
    "RejectedAdHocExecutePromptEvent",
    "RejectedExecutePromptEvent",
    "RejectedExecutePromptResponse",
    "RejectedExecuteWorkflowWorkflowResultEvent",
    "RejectedPromptExecutionMeta",
    "RejectedWorkflowNodeResultEvent",
    "ReleaseCreatedBy",
    "ReleaseEnvironment",
    "ReleaseReleaseTag",
    "ReleaseReviewReviewer",
    "ReleaseReviewState",
    "ReleaseTagRelease",
    "ReleaseTagSource",
    "ReplaceTestSuiteTestCaseRequest",
    "RichTextChildBlock",
    "RichTextPromptBlock",
    "SandboxScenario",
    "ScenarioInput",
    "ScenarioInputAudioVariableValue",
    "ScenarioInputChatHistoryVariableValue",
    "ScenarioInputDocumentVariableValue",
    "ScenarioInputImageVariableValue",
    "ScenarioInputJsonVariableValue",
    "ScenarioInputStringVariableValue",
    "ScenarioInputVideoVariableValue",
    "ScheduledTriggerContext",
    "SearchFiltersRequest",
    "SearchNodeResult",
    "SearchNodeResultData",
    "SearchRequestOptionsRequest",
    "SearchResponse",
    "SearchResult",
    "SearchResultDocument",
    "SearchResultDocumentRequest",
    "SearchResultMergingRequest",
    "SearchResultMeta",
    "SearchResultMetaRequest",
    "SearchResultRequest",
    "SearchResultsInput",
    "SearchResultsVellumValue",
    "SearchResultsVellumValueRequest",
    "SearchWeightsRequest",
    "SecretTypeEnum",
    "SentenceChunkerConfig",
    "SentenceChunkerConfigRequest",
    "SentenceChunking",
    "SentenceChunkingRequest",
    "SlimComposioToolDefinition",
    "SlimDeploymentRead",
    "SlimDocument",
    "SlimDocumentDocumentToDocumentIndex",
    "SlimIntegrationAuthConfigRead",
    "SlimIntegrationRead",
    "SlimReleaseReview",
    "SlimWorkflowDeployment",
    "SlimWorkflowExecutionRead",
    "SpanLink",
    "SpanLinkTypeEnum",
    "StreamingAdHocExecutePromptEvent",
    "StreamingExecutePromptEvent",
    "StreamingPromptExecutionMeta",
    "StreamingWorkflowNodeResultEvent",
    "StringChatMessageContent",
    "StringChatMessageContentRequest",
    "StringInput",
    "StringInputRequest",
    "StringVellumValue",
    "StringVellumValueRequest",
    "SubmitCompletionActualRequest",
    "SubmitWorkflowExecutionActualRequest",
    "SubworkflowNodeResult",
    "SubworkflowNodeResultData",
    "TemplatingNodeArrayResult",
    "TemplatingNodeChatHistoryResult",
    "TemplatingNodeErrorResult",
    "TemplatingNodeFunctionCallResult",
    "TemplatingNodeJsonResult",
    "TemplatingNodeNumberResult",
    "TemplatingNodeResult",
    "TemplatingNodeResultData",
    "TemplatingNodeResultOutput",
    "TemplatingNodeSearchResultsResult",
    "TemplatingNodeStringResult",
    "TerminalNodeArrayResult",
    "TerminalNodeChatHistoryResult",
    "TerminalNodeErrorResult",
    "TerminalNodeFunctionCallResult",
    "TerminalNodeJsonResult",
    "TerminalNodeNumberResult",
    "TerminalNodeResult",
    "TerminalNodeResultData",
    "TerminalNodeResultOutput",
    "TerminalNodeSearchResultsResult",
    "TerminalNodeStringResult",
    "TestCaseArrayVariableValue",
    "TestCaseAudioVariableValue",
    "TestCaseChatHistoryVariableValue",
    "TestCaseDocumentVariableValue",
    "TestCaseErrorVariableValue",
    "TestCaseFunctionCallVariableValue",
    "TestCaseImageVariableValue",
    "TestCaseJsonVariableValue",
    "TestCaseNumberVariableValue",
    "TestCaseSearchResultsVariableValue",
    "TestCaseStringVariableValue",
    "TestCaseVariableValue",
    "TestCaseVideoVariableValue",
    "TestSuiteRunDeploymentReleaseTagExecConfig",
    "TestSuiteRunDeploymentReleaseTagExecConfigData",
    "TestSuiteRunDeploymentReleaseTagExecConfigDataRequest",
    "TestSuiteRunDeploymentReleaseTagExecConfigRequest",
    "TestSuiteRunExecConfig",
    "TestSuiteRunExecConfigRequest",
    "TestSuiteRunExecution",
    "TestSuiteRunExecutionArrayOutput",
    "TestSuiteRunExecutionChatHistoryOutput",
    "TestSuiteRunExecutionErrorOutput",
    "TestSuiteRunExecutionFunctionCallOutput",
    "TestSuiteRunExecutionJsonOutput",
    "TestSuiteRunExecutionMetricDefinition",
    "TestSuiteRunExecutionMetricResult",
    "TestSuiteRunExecutionNumberOutput",
    "TestSuiteRunExecutionOutput",
    "TestSuiteRunExecutionSearchResultsOutput",
    "TestSuiteRunExecutionStringOutput",
    "TestSuiteRunExternalExecConfig",
    "TestSuiteRunExternalExecConfigData",
    "TestSuiteRunExternalExecConfigDataRequest",
    "TestSuiteRunExternalExecConfigRequest",
    "TestSuiteRunMetricArrayOutput",
    "TestSuiteRunMetricErrorOutput",
    "TestSuiteRunMetricJsonOutput",
    "TestSuiteRunMetricNumberOutput",
    "TestSuiteRunMetricOutput",
    "TestSuiteRunMetricStringOutput",
    "TestSuiteRunProgress",
    "TestSuiteRunPromptSandboxExecConfigDataRequest",
    "TestSuiteRunPromptSandboxExecConfigRequest",
    "TestSuiteRunPromptSandboxHistoryItemExecConfig",
    "TestSuiteRunPromptSandboxHistoryItemExecConfigData",
    "TestSuiteRunPromptSandboxHistoryItemExecConfigDataRequest",
    "TestSuiteRunPromptSandboxHistoryItemExecConfigRequest",
    "TestSuiteRunRead",
    "TestSuiteRunState",
    "TestSuiteRunTestSuite",
    "TestSuiteRunWorkflowReleaseTagExecConfig",
    "TestSuiteRunWorkflowReleaseTagExecConfigData",
    "TestSuiteRunWorkflowReleaseTagExecConfigDataRequest",
    "TestSuiteRunWorkflowReleaseTagExecConfigRequest",
    "TestSuiteRunWorkflowSandboxExecConfigDataRequest",
    "TestSuiteRunWorkflowSandboxExecConfigRequest",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfig",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfigData",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfigDataRequest",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfigRequest",
    "TestSuiteTestCase",
    "TestSuiteTestCaseBulkOperationRequest",
    "TestSuiteTestCaseBulkResult",
    "TestSuiteTestCaseCreateBulkOperationRequest",
    "TestSuiteTestCaseCreatedBulkResult",
    "TestSuiteTestCaseCreatedBulkResultData",
    "TestSuiteTestCaseDeleteBulkOperationDataRequest",
    "TestSuiteTestCaseDeleteBulkOperationRequest",
    "TestSuiteTestCaseDeletedBulkResult",
    "TestSuiteTestCaseDeletedBulkResultData",
    "TestSuiteTestCaseRejectedBulkResult",
    "TestSuiteTestCaseReplaceBulkOperationRequest",
    "TestSuiteTestCaseReplacedBulkResult",
    "TestSuiteTestCaseReplacedBulkResultData",
    "TestSuiteTestCaseUpsertBulkOperationRequest",
    "ThinkingVellumValue",
    "ThinkingVellumValueRequest",
    "TokenOverlappingWindowChunkerConfig",
    "TokenOverlappingWindowChunkerConfigRequest",
    "TokenOverlappingWindowChunking",
    "TokenOverlappingWindowChunkingRequest",
    "TooManyRequestsError",
    "UnauthorizedError",
    "UnitEnum",
    "UploadDocumentResponse",
    "UpsertTestSuiteTestCaseRequest",
    "VariablePromptBlock",
    "Vellum",
    "VellumAudio",
    "VellumAudioRequest",
    "VellumCodeResourceDefinition",
    "VellumDocument",
    "VellumDocumentRequest",
    "VellumEnvironment",
    "VellumError",
    "VellumErrorCodeEnum",
    "VellumErrorRequest",
    "VellumImage",
    "VellumImageRequest",
    "VellumNodeExecutionEvent",
    "VellumSdkError",
    "VellumSdkErrorCodeEnum",
    "VellumSecret",
    "VellumSpan",
    "VellumValue",
    "VellumValueLogicalConditionGroupRequest",
    "VellumValueLogicalConditionRequest",
    "VellumValueLogicalExpressionRequest",
    "VellumValueRequest",
    "VellumVariable",
    "VellumVariableExtensions",
    "VellumVariableType",
    "VellumVideo",
    "VellumVideoRequest",
    "VellumWorkflowExecutionEvent",
    "VideoChatMessageContent",
    "VideoChatMessageContentRequest",
    "VideoInput",
    "VideoInputRequest",
    "VideoPromptBlock",
    "VideoVellumValue",
    "VideoVellumValueRequest",
    "WorkflowDeploymentDisplayData",
    "WorkflowDeploymentEventExecutionsResponse",
    "WorkflowDeploymentHistoryItem",
    "WorkflowDeploymentParentContext",
    "WorkflowDeploymentRead",
    "WorkflowDeploymentRelease",
    "WorkflowDeploymentReleaseWorkflowDeployment",
    "WorkflowDeploymentReleaseWorkflowVersion",
    "WorkflowDeploymentsListRequestStatus",
    "WorkflowDisplayIcon",
    "WorkflowError",
    "WorkflowEvent",
    "WorkflowEventError",
    "WorkflowEventExecutionRead",
    "WorkflowExecutionActual",
    "WorkflowExecutionActualChatHistoryRequest",
    "WorkflowExecutionActualJsonRequest",
    "WorkflowExecutionActualStringRequest",
    "WorkflowExecutionDetail",
    "WorkflowExecutionEventErrorCode",
    "WorkflowExecutionEventType",
    "WorkflowExecutionFulfilledBody",
    "WorkflowExecutionFulfilledEvent",
    "WorkflowExecutionInitiatedBody",
    "WorkflowExecutionInitiatedEvent",
    "WorkflowExecutionNodeResultEvent",
    "WorkflowExecutionPausedBody",
    "WorkflowExecutionPausedEvent",
    "WorkflowExecutionRejectedBody",
    "WorkflowExecutionRejectedEvent",
    "WorkflowExecutionResumedBody",
    "WorkflowExecutionResumedEvent",
    "WorkflowExecutionSnapshottedBody",
    "WorkflowExecutionSnapshottedEvent",
    "WorkflowExecutionSpan",
    "WorkflowExecutionSpanAttributes",
    "WorkflowExecutionStreamingBody",
    "WorkflowExecutionStreamingEvent",
    "WorkflowExecutionUsageCalculationError",
    "WorkflowExecutionUsageCalculationErrorCodeEnum",
    "WorkflowExecutionUsageCalculationFulfilledBody",
    "WorkflowExecutionUsageResult",
    "WorkflowExecutionViewOnlineEvalMetricResult",
    "WorkflowExecutionWorkflowResultEvent",
    "WorkflowExpandMetaRequest",
    "WorkflowInitializationError",
    "WorkflowInput",
    "WorkflowNodeResultData",
    "WorkflowNodeResultEvent",
    "WorkflowNodeResultEventState",
    "WorkflowOutput",
    "WorkflowOutputArray",
    "WorkflowOutputChatHistory",
    "WorkflowOutputError",
    "WorkflowOutputFunctionCall",
    "WorkflowOutputImage",
    "WorkflowOutputJson",
    "WorkflowOutputNumber",
    "WorkflowOutputSearchResults",
    "WorkflowOutputString",
    "WorkflowParentContext",
    "WorkflowPushDeploymentConfigRequest",
    "WorkflowPushExecConfig",
    "WorkflowPushResponse",
    "WorkflowReleaseTagRead",
    "WorkflowReleaseTagWorkflowDeploymentHistoryItem",
    "WorkflowRequestAudioInputRequest",
    "WorkflowRequestChatHistoryInputRequest",
    "WorkflowRequestDocumentInputRequest",
    "WorkflowRequestImageInputRequest",
    "WorkflowRequestInputRequest",
    "WorkflowRequestJsonInputRequest",
    "WorkflowRequestNumberInputRequest",
    "WorkflowRequestStringInputRequest",
    "WorkflowRequestVideoInputRequest",
    "WorkflowResolvedState",
    "WorkflowResultEvent",
    "WorkflowResultEventOutputData",
    "WorkflowResultEventOutputDataArray",
    "WorkflowResultEventOutputDataChatHistory",
    "WorkflowResultEventOutputDataError",
    "WorkflowResultEventOutputDataFunctionCall",
    "WorkflowResultEventOutputDataJson",
    "WorkflowResultEventOutputDataNumber",
    "WorkflowResultEventOutputDataSearchResults",
    "WorkflowResultEventOutputDataString",
    "WorkflowSandboxDisplayData",
    "WorkflowSandboxExample",
    "WorkflowSandboxParentContext",
    "WorkflowStreamEvent",
    "WorkspaceDisplayConfig",
    "WorkspaceRead",
    "WorkspaceSecretRead",
    "__version__",
    "ad_hoc",
    "container_images",
    "deployments",
    "document_indexes",
    "documents",
    "environments",
    "events",
    "folder_entities",
    "integration_auth_configs",
    "integration_providers",
    "integrations",
    "metric_definitions",
    "ml_models",
    "organizations",
    "prompts",
    "sandboxes",
    "test_suite_runs",
    "test_suites",
    "workflow_deployments",
    "workflow_executions",
    "workflow_sandboxes",
    "workflows",
    "workspace_secrets",
    "workspaces",
]



"""Tests for IntegrationTrigger serialization."""

from vellum.workflows import BaseWorkflow
from vellum.workflows.inputs.base import BaseInputs
from vellum.workflows.nodes.bases.base import BaseNode
from vellum.workflows.state.base import BaseState
from vellum.workflows.triggers.integration import IntegrationTrigger
from vellum_ee.workflows.display.workflows.get_vellum_workflow_display_class import get_workflow_display


def test_vellum_integration_trigger_serialization():
    """IntegrationTrigger subclass serializes with class name and module path."""

    # Create a custom IntegrationTrigger subclass
    class SlackMessageTrigger(IntegrationTrigger):
        """Custom Slack message trigger."""

        message: str
        channel: str
        user: str

        class Config:
            provider = "COMPOSIO"
            integration_name = "SLACK"
            slug = "slack_new_message"

    class ProcessNode(BaseNode):
        """Node that processes the trigger."""

        class Outputs(BaseNode.Outputs):
            result = SlackMessageTrigger.message

        def run(self) -> Outputs:
            return self.Outputs()

    class TestWorkflow(BaseWorkflow[BaseInputs, BaseState]):
        graph = SlackMessageTrigger >> ProcessNode

    result: dict = get_workflow_display(workflow_class=TestWorkflow).serialize()

    # Verify triggers field exists
    assert "triggers" in result
    triggers = result["triggers"]
    assert isinstance(triggers, list)
    assert len(triggers) == 1

    trigger = triggers[0]
    assert isinstance(trigger, dict)
    assert trigger["type"] == "INTEGRATION"

    # Check that attributes are serialized
    assert "attributes" in trigger
    attributes = trigger["attributes"]
    assert isinstance(attributes, list)
    assert len(attributes) == 3

    attribute_names = {attr["name"] for attr in attributes if isinstance(attr, dict)}
    assert attribute_names == {"message", "channel", "user"}

    # RED: These assertions should fail because we haven't implemented class_name and module_path yet
    assert "class_name" in trigger, "Trigger should include class_name for codegen"
    assert trigger["class_name"] == "SlackMessageTrigger"

    assert "module_path" in trigger, "Trigger should include module_path for codegen"
    # The module path should be a list of strings representing the module hierarchy
    module_path = trigger["module_path"]
    assert isinstance(module_path, list)
    assert all(isinstance(part, str) for part in module_path)


def test_vellum_integration_trigger_id_consistency():
    """Validates trigger and attribute IDs match between definitions and references."""

    class SlackMessageTrigger(IntegrationTrigger):
        message: str
        channel: str

        class Config:
            provider = "COMPOSIO"
            integration_name = "SLACK"
            slug = "slack_new_message"

    class ProcessNode(BaseNode):
        class Outputs(BaseNode.Outputs):
            msg_output = SlackMessageTrigger.message
            channel_output = SlackMessageTrigger.channel

        def run(self) -> Outputs:
            return self.Outputs()

    class TestWorkflow(BaseWorkflow[BaseInputs, BaseState]):
        graph = SlackMessageTrigger >> ProcessNode

    result: dict = get_workflow_display(workflow_class=TestWorkflow).serialize()

    # Get trigger definition IDs
    triggers = result["triggers"]
    assert isinstance(triggers, list)
    trigger = triggers[0]
    assert isinstance(trigger, dict)
    trigger_id = trigger["id"]
    trigger_attributes = trigger["attributes"]
    assert isinstance(trigger_attributes, list)
    trigger_attrs = {attr["name"]: attr["id"] for attr in trigger_attributes if isinstance(attr, dict)}

    # Find node with trigger attribute references
    workflow_raw_data = result["workflow_raw_data"]
    assert isinstance(workflow_raw_data, dict)
    nodes = workflow_raw_data["nodes"]
    assert isinstance(nodes, list)
    process_node = next(
        (
            n
            for n in nodes
            if isinstance(n, dict)
            and isinstance(n.get("outputs"), list)
            and any(
                isinstance(o, dict)
                and isinstance(o.get("value"), dict)
                and o.get("value", {}).get("type") == "TRIGGER_ATTRIBUTE"
                for o in n.get("outputs", [])
            )
        ),
        None,
    )
    assert process_node, "No node found with trigger attribute references"

    # Validate all trigger attribute references have matching IDs
    attr_mapping = {"msg_output": "message", "channel_output": "channel"}
    outputs = process_node["outputs"]
    assert isinstance(outputs, list)
    for output in outputs:
        if not isinstance(output, dict):
            continue
        value = output.get("value", {})
        if not isinstance(value, dict):
            continue
        if value.get("type") == "TRIGGER_ATTRIBUTE":
            assert value["trigger_id"] == trigger_id, "Trigger ID mismatch"

            output_name = output["name"]
            expected_attr = attr_mapping[output_name]
            expected_id = trigger_attrs[expected_attr]
            assert value["attribute_id"] == expected_id, f"Attribute ID mismatch for {expected_attr}"


def test_trigger_module_paths_are_canonical():
    """Validates trigger module_path and class_name for consistent codegen."""

    class TestSlackTrigger(IntegrationTrigger):
        message: str

        class Config:
            provider = "COMPOSIO"
            integration_name = "SLACK"
            slug = "test_slack_trigger"

    class SimpleNode(BaseNode):
        class Outputs(BaseNode.Outputs):
            result = TestSlackTrigger.message

        def run(self) -> Outputs:
            return self.Outputs()

    class TestWorkflow(BaseWorkflow[BaseInputs, BaseState]):
        graph = TestSlackTrigger >> SimpleNode

    result = get_workflow_display(workflow_class=TestWorkflow).serialize()

    triggers = result["triggers"]
    assert isinstance(triggers, list)
    trigger = triggers[0]
    assert isinstance(trigger, dict)
    assert trigger["type"] == "INTEGRATION"

    module_path = trigger["module_path"]
    assert isinstance(module_path, list)
    assert all(isinstance(part, str) for part in module_path)
    assert module_path == __name__.split(".")

    assert trigger["class_name"] == "TestSlackTrigger"


def test_integration_trigger_no_entrypoint_node():
    """IntegrationTrigger workflows now create ENTRYPOINT nodes and route edges through them."""

    class SlackMessageTrigger(IntegrationTrigger):
        message: str

        class Config:
            provider = "COMPOSIO"
            integration_name = "SLACK"
            slug = "slack_message"

    class ProcessNode(BaseNode):
        pass

    class TestWorkflow(BaseWorkflow[BaseInputs, BaseState]):
        graph = SlackMessageTrigger >> ProcessNode

    result = get_workflow_display(workflow_class=TestWorkflow).serialize()

    # Get trigger ID
    triggers = result["triggers"]
    assert isinstance(triggers, list)
    assert len(triggers) == 1
    trigger = triggers[0]
    assert isinstance(trigger, dict)
    trigger_id = trigger["id"]

    # Verify trigger has source_handle_id matching trigger_id
    assert "source_handle_id" in trigger, "Trigger should have source_handle_id"
    assert trigger["source_handle_id"] == trigger_id, "source_handle_id should match trigger_id"

    # Verify ENTRYPOINT node exists
    workflow_raw_data = result["workflow_raw_data"]
    assert isinstance(workflow_raw_data, dict)
    nodes = workflow_raw_data["nodes"]
    assert isinstance(nodes, list)
    entrypoint_nodes = [n for n in nodes if isinstance(n, dict) and n.get("type") == "ENTRYPOINT"]
    assert len(entrypoint_nodes) == 1, "IntegrationTrigger workflows should have an ENTRYPOINT node"

    entrypoint_node = entrypoint_nodes[0]
    assert isinstance(entrypoint_node, dict)
    entrypoint_node_id = entrypoint_node["id"]

    edges = workflow_raw_data["edges"]
    assert isinstance(edges, list)
    entrypoint_edges = [e for e in edges if isinstance(e, dict) and e.get("source_node_id") == entrypoint_node_id]
    assert len(entrypoint_edges) == 0

    # Verify edges use trigger ID as sourceNodeId (not ENTRYPOINT)
    trigger_edges = [e for e in edges if isinstance(e, dict) and e.get("source_node_id") == trigger_id]
    assert len(trigger_edges) > 0, "Should have edges from trigger ID"

    # Verify the edge connects trigger to first node
    # ProcessNode should be the only non-terminal, non-entrypoint node
    process_nodes = [n for n in nodes if isinstance(n, dict) and n.get("type") not in ("TERMINAL", "ENTRYPOINT")]
    assert len(process_nodes) > 0, "Should have at least one process node"
    process_node = process_nodes[0]
    assert isinstance(process_node, dict)
    process_node_id = process_node["id"]

    trigger_to_process_edge = next(
        (e for e in trigger_edges if isinstance(e, dict) and e.get("target_node_id") == process_node_id),
        None,
    )
    assert trigger_to_process_edge is not None, "Should have edge from trigger to ProcessNode"
    assert isinstance(trigger_to_process_edge, dict)
    assert trigger_to_process_edge["source_node_id"] == trigger_id
    assert trigger_to_process_edge["target_node_id"] == process_node_id


def test_integration_trigger_serialization_display_data():
    """IntegrationTrigger with Display class serializes all display attributes correctly."""

    # GIVEN an integration trigger with comprehensive Display attributes
    class SlackMessageTriggerWithDisplay(IntegrationTrigger):
        message: str
        channel: str
        user: str

        class Config:
            provider = "COMPOSIO"
            integration_name = "SLACK"
            slug = "slack_new_message"

        class Display(IntegrationTrigger.Display):
            label = "Slack Message"
            x = 150.5
            y = 250.75
            z_index = 5
            icon = "vellum:icon:integrations/COMPOSIO/SLACK"
            color = "#4A154B"

    class ProcessNode(BaseNode):
        class Outputs(BaseNode.Outputs):
            result = SlackMessageTriggerWithDisplay.message

        def run(self) -> Outputs:
            return self.Outputs()

    # AND a workflow that uses the trigger
    class TestWorkflow(BaseWorkflow[BaseInputs, BaseState]):
        graph = SlackMessageTriggerWithDisplay >> ProcessNode

    # WHEN we serialize the workflow
    result: dict = get_workflow_display(workflow_class=TestWorkflow).serialize()

    # THEN we get the expected trigger
    assert "triggers" in result
    triggers = result["triggers"]
    assert isinstance(triggers, list)
    assert len(triggers) == 1

    trigger = triggers[0]
    assert isinstance(trigger, dict)
    assert trigger["type"] == "INTEGRATION"

    # AND display_data is serialized with icon and color
    assert "display_data" in trigger
    display_data = trigger["display_data"]
    assert isinstance(display_data, dict)
    assert display_data["icon"] == "vellum:icon:integrations/COMPOSIO/SLACK"
    assert display_data["color"] == "#4A154B"
